
# line 2 "dgn_comp.y"
/* NetHack 3.6  dgn_comp.y	$NHDT-Date: 1432512785 2015/05/25 00:13:05 $  $NHDT-Branch: master $:$NHDT-Revision: 1.8 $ */
/*	Copyright (c) 1989 by Jean-Christophe Collet */
/*	Copyright (c) 1990 by M. Stephenson				  */
/* NetHack may be freely redistributed.  See license for details. */

/*
 * This file contains the Dungeon Compiler code
 */

/* In case we're using bison in AIX.  This definition must be
 * placed before any other C-language construct in the file
 * excluding comments and preprocessor directives (thanks IBM
 * for this wonderful feature...).
 *
 * Note: some cpps barf on this 'undefined control' (#pragma).
 * Addition of the leading space seems to prevent barfage for now,
 * and AIX will still see the directive in its non-standard locale.
 */

#ifdef _AIX
 #pragma alloca		/* keep leading space! */
#endif

#include "config.h"
#include "date.h"
#include "dgn_file.h"

void FDECL(yyerror, (const char *));
void FDECL(yywarning, (const char *));
int NDECL(yylex);
int NDECL(yyparse);
int FDECL(getchain, (char *));
int NDECL(check_dungeon);
int NDECL(check_branch);
int NDECL(check_level);
void NDECL(init_dungeon);
void NDECL(init_branch);
void NDECL(init_level);
void NDECL(output_dgn);

#define Free(ptr)		free((genericptr_t)ptr)

#ifdef AMIGA
# undef	printf
#ifndef	LATTICE
# define    memset(addr,val,len)    setmem(addr,len,val)
#endif
#endif

#define ERR		(-1)

static struct couple couple;
static struct tmpdungeon tmpdungeon[MAXDUNGEON];
static struct tmplevel tmplevel[LEV_LIMIT];
static struct tmpbranch tmpbranch[BRANCH_LIMIT];

static int in_dungeon = 0, n_dgns = -1, n_levs = -1, n_brs = -1;

extern int fatal_error;
extern const char *fname;
extern FILE *yyin, *yyout;	/* from dgn_lex.c */


# line 66 "dgn_comp.y"
typedef union 
{
	int	i;
	char*	str;
} YYSTYPE;
# define INTEGER 257
# define A_DUNGEON 258
# define BRANCH 259
# define CHBRANCH 260
# define LEVEL 261
# define RNDLEVEL 262
# define CHLEVEL 263
# define RNDCHLEVEL 264
# define UP_OR_DOWN 265
# define PROTOFILE 266
# define DESCRIPTION 267
# define DESCRIPTOR 268
# define LEVELDESC 269
# define ALIGNMENT 270
# define LEVALIGN 271
# define ENTRY 272
# define STAIR 273
# define NO_UP 274
# define NO_DOWN 275
# define PORTAL 276
# define STRING 277
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern short yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 432 "dgn_comp.y"


void
init_dungeon()
{
	if(++n_dgns > MAXDUNGEON) {
	    (void) fprintf(stderr, "FATAL - Too many dungeons (limit: %d).\n",
		    MAXDUNGEON);
	    (void) fprintf(stderr, "To increase the limit edit MAXDUNGEON in global.h\n");
	    exit(EXIT_FAILURE);
	}

	in_dungeon = 1;
	tmpdungeon[n_dgns].lev.base = 0;
	tmpdungeon[n_dgns].lev.rand = 0;
	tmpdungeon[n_dgns].chance = 100;
	Strcpy(tmpdungeon[n_dgns].name, "");
	Strcpy(tmpdungeon[n_dgns].protoname, "");
	tmpdungeon[n_dgns].flags = 0;
	tmpdungeon[n_dgns].levels = 0;
	tmpdungeon[n_dgns].branches = 0;
	tmpdungeon[n_dgns].entry_lev = 0;
}

void
init_level()
{
	if(++n_levs > LEV_LIMIT) {

		yyerror("FATAL - Too many special levels defined.");
		exit(EXIT_FAILURE);
	}
	tmplevel[n_levs].lev.base = 0;
	tmplevel[n_levs].lev.rand = 0;
	tmplevel[n_levs].chance = 100;
	tmplevel[n_levs].rndlevs = 0;
	tmplevel[n_levs].flags = 0;
	Strcpy(tmplevel[n_levs].name, "");
	tmplevel[n_levs].chain = -1;
}

void
init_branch()
{
	if(++n_brs > BRANCH_LIMIT) {

		yyerror("FATAL - Too many special levels defined.");
		exit(EXIT_FAILURE);
	}
	tmpbranch[n_brs].lev.base = 0;
	tmpbranch[n_brs].lev.rand = 0;
	Strcpy(tmpbranch[n_brs].name, "");
	tmpbranch[n_brs].chain = -1;
}

int
getchain(s)
	char	*s;
{
	int i;

	if(strlen(s)) {

	    for(i = n_levs - tmpdungeon[n_dgns].levels + 1; i <= n_levs; i++)
		if(!strcmp(tmplevel[i].name, s)) return i;

	    yyerror("Can't locate the specified chain level.");
	    return(-2);
	}
	return(-1);
}

/*
 *	Consistancy checking routines:
 *
 *	- A dungeon must have a unique name.
 *	- A dungeon must have a originating "branch" command
 *	  (except, of course, for the first dungeon).
 *	- A dungeon must have a proper depth (at least (1, 0)).
 */

int
check_dungeon()
{
	int i;

	for(i = 0; i < n_dgns; i++)
	    if(!strcmp(tmpdungeon[i].name, tmpdungeon[n_dgns].name)) {
		yyerror("Duplicate dungeon name.");
		return(0);
	    }

	if(n_dgns)
	  for(i = 0; i < n_brs - tmpdungeon[n_dgns].branches; i++) {
	    if(!strcmp(tmpbranch[i].name, tmpdungeon[n_dgns].name)) break;

	    if(i >= n_brs - tmpdungeon[n_dgns].branches) {
		yyerror("Dungeon cannot be reached.");
		return(0);
	    }
	  }

	if(tmpdungeon[n_dgns].lev.base <= 0 ||
	   tmpdungeon[n_dgns].lev.rand < 0) {
		yyerror("Invalid dungeon depth specified.");
		return(0);
	}
	return(1);	/* OK */
}

/*
 *	- A level must have a unique level name.
 *	- If chained, the level used as reference for the chain
 *	  must be in this dungeon, must be previously defined, and
 *	  the level chained from must be "non-probabilistic" (ie.
 *	  have a 100% chance of existing).
 */

int
check_level()
{
	int i;

	if(!in_dungeon) {
		yyerror("Level defined outside of dungeon.");
		return(0);
	}

	for(i = 0; i < n_levs; i++)
	    if(!strcmp(tmplevel[i].name, tmplevel[n_levs].name)) {
		yyerror("Duplicate level name.");
		return(0);
	    }

	if(tmplevel[i].chain == -2) {
		yyerror("Invaild level chain reference.");
		return(0);
	} else if(tmplevel[i].chain != -1) {	/* there is a chain */
	    /* KMH -- tmplevel[tmpbranch[i].chain].chance was in error */
	    if(tmplevel[tmplevel[i].chain].chance != 100) {
		yyerror("Level cannot chain from a probabilistic level.");
		return(0);
	    } else if(tmplevel[i].chain == n_levs) {
		yyerror("A level cannot chain to itself!");
		return(0);
	    }
	}
	return(1);	/* OK */
}

/*
 *	- A branch may not branch backwards - to avoid branch loops.
 *	- A branch name must be unique.
 *	  (ie. You can only have one entry point to each dungeon).
 *	- If chained, the level used as reference for the chain
 *	  must be in this dungeon, must be previously defined, and
 *	  the level chained from must be "non-probabilistic" (ie.
 *	  have a 100% chance of existing).
 */

int
check_branch()
{
	int i;

	if(!in_dungeon) {
		yyerror("Branch defined outside of dungeon.");
		return(0);
	}

	for(i = 0; i < n_dgns; i++)
	    if(!strcmp(tmpdungeon[i].name, tmpbranch[n_brs].name)) {

		yyerror("Reverse branching not allowed.");
		return(0);
	    }

	if(tmpbranch[i].chain == -2) {

		yyerror("Invaild branch chain reference.");
		return(0);
	} else if(tmpbranch[i].chain != -1) {	/* it is chained */

	    if(tmplevel[tmpbranch[i].chain].chance != 100) {
		yyerror("Branch cannot chain from a probabilistic level.");
		return(0);
	    }
	}
	return(1);	/* OK */
}

/*
 *	Output the dungon definition into a file.
 *
 *	The file will have the following format:
 *
 *	[ nethack version ID ]
 *	[ number of dungeons ]
 *	[ first dungeon struct ]
 *	[ levels for the first dungeon ]
 *	  ...
 *	[ branches for the first dungeon ]
 *	  ...
 *	[ second dungeon struct ]
 *	  ...
 */

void
output_dgn()
{
	int	nd, cl = 0, nl = 0,
		    cb = 0, nb = 0;
	static struct version_info version_data = {
			VERSION_NUMBER, VERSION_FEATURES,
			VERSION_SANITY1, VERSION_SANITY2, VERSION_SANITY3
	};

	if(++n_dgns <= 0) {
	    yyerror("FATAL - no dungeons were defined.");
	    exit(EXIT_FAILURE);
	}

	if (fwrite((char *)&version_data, sizeof version_data, 1, yyout) != 1) {
	    yyerror("FATAL - output failure.");
	    exit(EXIT_FAILURE);
	}

	(void) fwrite((char *)&n_dgns, sizeof(int), 1, yyout);
	for (nd = 0; nd < n_dgns; nd++) {
	    (void) fwrite((char *)&tmpdungeon[nd], sizeof(struct tmpdungeon),
							1, yyout);

	    nl += tmpdungeon[nd].levels;
	    for(; cl < nl; cl++)
		(void) fwrite((char *)&tmplevel[cl], sizeof(struct tmplevel),
							1, yyout);

	    nb += tmpdungeon[nd].branches;
	    for(; cb < nb; cb++)
		(void) fwrite((char *)&tmpbranch[cb], sizeof(struct tmpbranch),
							1, yyout);
	}
	/* apparently necessary for Think C 5.x, otherwise harmless */
	(void) fflush(yyout);
}

/*dgn_comp.y*/
short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 49
# define YYLAST 142
short yyact[]={

   8,  22,  23,  24,  25,  28,  29,  75,  21,  30,
  74,  26,  31,  27,  19,  60,  80,  81,  82,  83,
  92,  62,  56,  55,  52,  51,  50,  49,  48,  46,
  58,  57,  54,  53, 107, 104, 103, 102, 100,  99,
  96,  95,  84,  78,  77,  47,  91,  67,  79,  69,
  73,  72,  61,  59,  45,  44,  43,  42,  41,  40,
  39,  38,  37,  36,  35,  34,  33,  93,  90,  89,
  88,  71, 106, 105,  68,  70,   3,  13,  12,  32,
  18,  17,  16,  15,  14,  20,  11,  10,   9,   7,
   6,   5,   4,   2,   1,  76,   0,   0,   0,   0,
   0,   0,   0,   0,   0,  63,  64,   0,   0,  65,
  66,   0,   0,   0,   0,   0,   0,   0,   0,  85,
   0,   0,  86,  87,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,  94,   0,  97,  98,   0,   0,
   0, 101 };
short yypact[]={

-258,-1000,-258,-1000,-1000,-1000,-1000,-1000,   8,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,   7,
-1000,   6,   5,   4,   3,   2,   1,   0,  -1,  -2,
  -3,  -4,-1000,-248,-212,-249,-250,-251,-252,-253,
-235,-236,-254,-255,-237,-238,-262,-1000,-1000, -12,
-256,-262,-262,-1000,-1000,-262,-262,-1000,-1000,  34,
-1000,  35,  28, -13, -14,-267,-270,-213,-214,-257,
-215,  34,  35,  35,  27,  26,-1000,-1000,  24,-245,
-1000,-1000,-1000,-1000,  23,-257,-216,-217,  34,  34,
-218,-1000,-1000,-219,-245,-1000,-220,-221,-222,  32,
  31,-1000,-1000,-1000,-223,-1000,-1000,-1000 };
short yypgo[]={

   0,  95,  46,  48,  53,  94,  93,  76,  92,  91,
  90,  89,  47,  88,  87,  86,  85,  84,  83,  82,
  81,  80,  49,  78,  77 };
short yyr1[]={

   0,   5,   5,   6,   6,   7,   7,   7,   7,   8,
   1,   1,   9,   9,   9,  13,  14,  16,  16,  15,
  11,  11,  11,  11,  11,  17,  17,  18,  18,  19,
  19,  20,  20,  21,  21,  10,  10,  23,  24,   3,
   3,   3,   3,   3,   2,   2,   4,  22,  12 };
short yyr2[]={

   0,   0,   1,   1,   2,   1,   1,   1,   1,   6,
   0,   1,   1,   1,   1,   3,   1,   3,   3,   3,
   1,   1,   1,   1,   1,   6,   7,   7,   8,   3,
   3,   7,   8,   8,   9,   1,   1,   7,   8,   0,
   1,   1,   1,   1,   0,   1,   1,   5,   5 };
short yychk[]={

-1000,  -5,  -6,  -7,  -8,  -9, -10, -11, 258, -13,
 -14, -15, -23, -24, -17, -18, -19, -20, -21, 272,
 -16, 266, 259, 260, 261, 262, 269, 271, 263, 264,
 267, 270,  -7,  58,  58,  58,  58,  58,  58,  58,
  58,  58,  58,  58,  58,  58, 277, 257, 277, 277,
 277, 277, 277, 268, 268, 277, 277, 268, 268,  -4,
 277,  64, 277,  -4,  -4,  -4,  -4, -12,  40, -22,
  40,  43,  64,  64, 277, 277,  -1, 257, 257,  -3,
 273, 274, 275, 276, 257, -12, -22, -22,  43,  43,
  44,  -2, 265,  44,  -3, 257, 257, -12, -12, 257,
 257,  -2, 257, 257, 257,  41,  41, 257 };
short yydef[]={

   1,  -2,   2,   3,   5,   6,   7,   8,   0,  12,
  13,  14,  35,  36,  20,  21,  22,  23,  24,   0,
  16,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   4,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,  15,  19,   0,
   0,   0,   0,  29,  30,   0,   0,  17,  18,   0,
  46,   0,   0,   0,   0,   0,   0,  10,   0,  39,
   0,   0,   0,   0,   0,   0,   9,  11,   0,  44,
  40,  41,  42,  43,   0,  39,  25,   0,   0,   0,
   0,  37,  45,   0,  44,  27,  26,  31,   0,   0,
   0,  38,  28,  33,  32,  48,  47,  34 };
#ifndef lint
static char yaccpar_sccsid[] = "@(#)yaccpar	4.1	(Berkeley)	2/11/83";
#endif

#
# define YYFLAG -1000
# define YYERROR goto yyerrlab
# define YYACCEPT return(0)
# define YYABORT return(1)

/*	parser for yacc output	*/

#ifdef YYDEBUG
int yydebug = 0; /* 1 for debugging */
#endif
YYSTYPE yyv[YYMAXDEPTH]; /* where the values are stored */
int yychar = -1; /* current input token number */
int yynerrs = 0;  /* number of errors */
short yyerrflag = 0;  /* error recovery flag */

yyparse() {

	short yys[YYMAXDEPTH];
	short yyj, yym;
	register YYSTYPE *yypvt;
	register short yystate, *yyps, yyn;
	register YYSTYPE *yypv;
	register short *yyxi;

	yystate = 0;
	yychar = -1;
	yynerrs = 0;
	yyerrflag = 0;
	yyps= &yys[-1];
	yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

#ifdef YYDEBUG
	if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
#endif
		if( ++yyps> &yys[YYMAXDEPTH] ) { yyerror( "yacc stack overflow" ); return(1); }
		*yyps = yystate;
		++yypv;
		*yypv = yyval;

 yynewstate:

	yyn = yypact[yystate];

	if( yyn<= YYFLAG ) goto yydefault; /* simple state */

	if( yychar<0 ) if( (yychar=yylex())<0 ) yychar=0;
	if( (yyn += yychar)<0 || yyn >= YYLAST ) goto yydefault;

	if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
		yychar = -1;
		yyval = yylval;
		yystate = yyn;
		if( yyerrflag > 0 ) --yyerrflag;
		goto yystack;
		}

 yydefault:
	/* default state action */

	if( (yyn=yydef[yystate]) == -2 ) {
		if( yychar<0 ) if( (yychar=yylex())<0 ) yychar = 0;
		/* look through exception table */

		for( yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=yystate) ; yyxi += 2 ) ; /* VOID */

		while( *(yyxi+=2) >= 0 ){
			if( *yyxi == yychar ) break;
			}
		if( (yyn = yyxi[1]) < 0 ) return(0);   /* accept */
		}

	if( yyn == 0 ){ /* error */
		/* error ... attempt to resume parsing */

		switch( yyerrflag ){

		case 0:   /* brand new error */

			yyerror( "syntax error" );
		yyerrlab:
			++yynerrs;

		case 1:
		case 2: /* incompletely recovered error ... try again */

			yyerrflag = 3;

			/* find a state where "error" is a legal shift action */

			while ( yyps >= yys ) {
			   yyn = yypact[*yyps] + YYERRCODE;
			   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
			      yystate = yyact[yyn];  /* simulate a shift of "error" */
			      goto yystack;
			      }
			   yyn = yypact[*yyps];

			   /* the current yyps has no shift onn "error", pop stack */

#ifdef YYDEBUG
			   if( yydebug ) printf( "error recovery pops state %d, uncovers %d\n", *yyps, yyps[-1] );
#endif
			   --yyps;
			   --yypv;
			   }

			/* there is no state on the stack with an error shift ... abort */

	yyabort:
			return(1);


		case 3:  /* no shift yet; clobber input char */

#ifdef YYDEBUG
			if( yydebug ) printf( "error recovery discards char %d\n", yychar );
#endif

			if( yychar == 0 ) goto yyabort; /* don't discard EOF, quit */
			yychar = -1;
			goto yynewstate;   /* try again in the same state */

			}

		}

	/* reduction by production yyn */

#ifdef YYDEBUG
		if( yydebug ) printf("reduce %d\n",yyn);
#endif
		yyps -= yyr2[yyn];
		yypvt = yypv;
		yypv -= yyr2[yyn];
		yyval = yypv[1];
		yym=yyn;
			/* consult goto table to find next state */
		yyn = yyr1[yyn];
		yyj = yypgo[yyn] + *yyps + 1;
		if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn ) yystate = yyact[yypgo[yyn]];
		switch(yym){
			
case 2:
# line 83 "dgn_comp.y"
{
			output_dgn();
		  } break;
case 9:
# line 99 "dgn_comp.y"
{
			init_dungeon();
			Strcpy(tmpdungeon[n_dgns].name, yypvt[-3].str);
			tmpdungeon[n_dgns].boneschar = (char)yypvt[-2].i;
			tmpdungeon[n_dgns].lev.base = couple.base;
			tmpdungeon[n_dgns].lev.rand = couple.rand;
			tmpdungeon[n_dgns].chance = yypvt[-0].i;
			Free(yypvt[-3].str);
		  } break;
case 10:
# line 111 "dgn_comp.y"
{
			yyval.i = 0;
		  } break;
case 11:
# line 115 "dgn_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 15:
# line 126 "dgn_comp.y"
{
			tmpdungeon[n_dgns].entry_lev = yypvt[-0].i;
		  } break;
case 17:
# line 135 "dgn_comp.y"
{
			if(yypvt[-0].i <= TOWN || yypvt[-0].i >= D_ALIGN_CHAOTIC)
			    yyerror("Illegal description - ignoring!");
			else
			    tmpdungeon[n_dgns].flags |= yypvt[-0].i ;
		  } break;
case 18:
# line 142 "dgn_comp.y"
{
			if(yypvt[-0].i && yypvt[-0].i < D_ALIGN_CHAOTIC)
			    yyerror("Illegal alignment - ignoring!");
			else
			    tmpdungeon[n_dgns].flags |= yypvt[-0].i ;
		  } break;
case 19:
# line 151 "dgn_comp.y"
{
			Strcpy(tmpdungeon[n_dgns].protoname, yypvt[-0].str);
			Free(yypvt[-0].str);
		  } break;
case 25:
# line 165 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-3].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-2].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-3].str);
		  } break;
case 26:
# line 175 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-4].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-3].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-4].str);
		  } break;
case 27:
# line 188 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-4].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-3].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-0].i;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-4].str);
		  } break;
case 28:
# line 199 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-5].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-4].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-1].i;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-5].str);
		  } break;
case 29:
# line 213 "dgn_comp.y"
{
			if(yypvt[-0].i >= D_ALIGN_CHAOTIC)
			    yyerror("Illegal description - ignoring!");
			else
			    tmplevel[n_levs].flags |= yypvt[-0].i ;
		  } break;
case 30:
# line 220 "dgn_comp.y"
{
			if(yypvt[-0].i && yypvt[-0].i < D_ALIGN_CHAOTIC)
			    yyerror("Illegal alignment - ignoring!");
			else
			    tmplevel[n_levs].flags |= yypvt[-0].i ;
		  } break;
case 31:
# line 229 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-4].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-3].i;
			tmplevel[n_levs].chain = getchain(yypvt[-2].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-4].str);
			Free(yypvt[-2].str);
		  } break;
case 32:
# line 242 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-5].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-4].i;
			tmplevel[n_levs].chain = getchain(yypvt[-3].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-5].str);
			Free(yypvt[-3].str);
		  } break;
case 33:
# line 258 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-5].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-4].i;
			tmplevel[n_levs].chain = getchain(yypvt[-3].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-0].i;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-5].str);
			Free(yypvt[-3].str);
		  } break;
case 34:
# line 272 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-6].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-5].i;
			tmplevel[n_levs].chain = getchain(yypvt[-4].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-1].i;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-6].str);
			Free(yypvt[-4].str);
		  } break;
case 37:
# line 293 "dgn_comp.y"
{
			init_branch();
			Strcpy(tmpbranch[n_brs].name, yypvt[-4].str);
			tmpbranch[n_brs].lev.base = couple.base;
			tmpbranch[n_brs].lev.rand = couple.rand;
			tmpbranch[n_brs].type = yypvt[-1].i;
			tmpbranch[n_brs].up = yypvt[-0].i;
			if(!check_branch()) n_brs--;
			else tmpdungeon[n_dgns].branches++;
			Free(yypvt[-4].str);
		  } break;
case 38:
# line 307 "dgn_comp.y"
{
			init_branch();
			Strcpy(tmpbranch[n_brs].name, yypvt[-5].str);
			tmpbranch[n_brs].chain = getchain(yypvt[-4].str);
			tmpbranch[n_brs].lev.base = couple.base;
			tmpbranch[n_brs].lev.rand = couple.rand;
			tmpbranch[n_brs].type = yypvt[-1].i;
			tmpbranch[n_brs].up = yypvt[-0].i;
			if(!check_branch()) n_brs--;
			else tmpdungeon[n_dgns].branches++;
			Free(yypvt[-5].str);
			Free(yypvt[-4].str);
		  } break;
case 39:
# line 323 "dgn_comp.y"
{
			yyval.i = TBR_STAIR;	/* two way stair */
		  } break;
case 40:
# line 327 "dgn_comp.y"
{
			yyval.i = TBR_STAIR;	/* two way stair */
		  } break;
case 41:
# line 331 "dgn_comp.y"
{
			yyval.i = TBR_NO_UP;	/* no up staircase */
		  } break;
case 42:
# line 335 "dgn_comp.y"
{
			yyval.i = TBR_NO_DOWN;	/* no down staircase */
		  } break;
case 43:
# line 339 "dgn_comp.y"
{
			yyval.i = TBR_PORTAL;	/* portal connection */
		  } break;
case 44:
# line 345 "dgn_comp.y"
{
			yyval.i = 0;	/* defaults to down */
		  } break;
case 45:
# line 349 "dgn_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 46:
# line 355 "dgn_comp.y"
{
			char *p = yypvt[-0].str;
			if (strlen(p) != 1) {
			    if (strcmp(p, "none") != 0)
		   yyerror("Bones marker must be a single char, or \"none\"!");
			    *p = '\0';
			}
			yyval.i = *p;
			Free(p);
		  } break;
case 47:
# line 385 "dgn_comp.y"
{
			if (yypvt[-3].i < -MAXLEVEL || yypvt[-3].i > MAXLEVEL) {
			    yyerror("Abs base out of dlevel range - zeroing!");
			    couple.base = couple.rand = 0;
			} else if (yypvt[-1].i < -1 ||
				((yypvt[-3].i < 0) ? (MAXLEVEL + yypvt[-3].i + yypvt[-1].i + 1) > MAXLEVEL :
					(yypvt[-3].i + yypvt[-1].i) > MAXLEVEL)) {
			    yyerror("Abs range out of dlevel range - zeroing!");
			    couple.base = couple.rand = 0;
			} else {
			    couple.base = yypvt[-3].i;
			    couple.rand = yypvt[-1].i;
			}
		  } break;
case 48:
# line 422 "dgn_comp.y"
{
			if (yypvt[-3].i < -MAXLEVEL || yypvt[-3].i > MAXLEVEL) {
			    yyerror("Rel base out of dlevel range - zeroing!");
			    couple.base = couple.rand = 0;
			} else {
			    couple.base = yypvt[-3].i;
			    couple.rand = yypvt[-1].i;
			}
		  } break; 
		}
		goto yystack;  /* stack new state and value */

	}
