
# line 2 "lev_comp.y"
/* NetHack 3.6  lev_comp.y	$NHDT-Date: 1543371691 2018/11/28 02:21:31 $  $NHDT-Branch: NetHack-3.6.2-beta01 $:$NHDT-Revision: 1.22 $ */
/*      Copyright (c) 1989 by Jean-Christophe Collet */
/* NetHack may be freely redistributed.  See license for details. */

/*
 * This file contains the Level Compiler code
 * It may handle special mazes & special room-levels
 */

/* In case we're using bison in AIX.  This definition must be
 * placed before any other C-language construct in the file
 * excluding comments and preprocessor directives (thanks IBM
 * for this wonderful feature...).
 *
 * Note: some cpps barf on this 'undefined control' (#pragma).
 * Addition of the leading space seems to prevent barfage for now,
 * and AIX will still see the directive.
 */
#ifdef _AIX
 #pragma alloca         /* keep leading space! */
#endif

#define SPEC_LEV    /* for USE_OLDARGS (sp_lev.h) */
#include "hack.h"
#include "sp_lev.h"

#define ERR             (-1)
/* many types of things are put in chars for transference to NetHack.
 * since some systems will use signed chars, limit everybody to the
 * same number for portability.
 */
#define MAX_OF_TYPE     128

#define MAX_NESTED_IFS   20
#define MAX_SWITCH_CASES 20

#define New(type) \
        (type *) memset((genericptr_t) alloc(sizeof (type)), 0, sizeof (type))
#define NewTab(type, size)      (type **) alloc(sizeof (type *) * size)
#define Free(ptr)               free((genericptr_t) ptr)

extern void VDECL(lc_error, (const char *, ...));
extern void VDECL(lc_warning, (const char *, ...));
extern void FDECL(yyerror, (const char *));
extern void FDECL(yywarning, (const char *));
extern int NDECL(yylex);
int NDECL(yyparse);

extern int FDECL(get_floor_type, (CHAR_P));
extern int FDECL(get_room_type, (char *));
extern int FDECL(get_trap_type, (char *));
extern int FDECL(get_monster_id, (char *,CHAR_P));
extern int FDECL(get_object_id, (char *,CHAR_P));
extern boolean FDECL(check_monster_char, (CHAR_P));
extern boolean FDECL(check_object_char, (CHAR_P));
extern char FDECL(what_map_char, (CHAR_P));
extern void FDECL(scan_map, (char *, sp_lev *));
extern void FDECL(add_opcode, (sp_lev *, int, genericptr_t));
extern genericptr_t FDECL(get_last_opcode_data1, (sp_lev *, int));
extern genericptr_t FDECL(get_last_opcode_data2, (sp_lev *, int, int));
extern boolean FDECL(check_subrooms, (sp_lev *));
extern boolean FDECL(write_level_file, (char *,sp_lev *));
extern struct opvar *FDECL(set_opvar_int, (struct opvar *, long));
extern void VDECL(add_opvars, (sp_lev *, const char *, ...));
extern void FDECL(start_level_def, (sp_lev * *, char *));

extern struct lc_funcdefs *FDECL(funcdef_new, (long,char *));
extern void FDECL(funcdef_free_all, (struct lc_funcdefs *));
extern struct lc_funcdefs *FDECL(funcdef_defined, (struct lc_funcdefs *,
                                                   char *, int));
extern char *FDECL(funcdef_paramtypes, (struct lc_funcdefs *));
extern char *FDECL(decode_parm_str, (char *));

extern struct lc_vardefs *FDECL(vardef_new, (long,char *));
extern void FDECL(vardef_free_all, (struct lc_vardefs *));
extern struct lc_vardefs *FDECL(vardef_defined, (struct lc_vardefs *,
                                                 char *, int));

extern void NDECL(break_stmt_start);
extern void FDECL(break_stmt_end, (sp_lev *));
extern void FDECL(break_stmt_new, (sp_lev *, long));

extern void FDECL(splev_add_from, (sp_lev *, sp_lev *));

extern void FDECL(check_vardef_type, (struct lc_vardefs *, char *, long));
extern void FDECL(vardef_used, (struct lc_vardefs *, char *));
extern struct lc_vardefs *FDECL(add_vardef_type, (struct lc_vardefs *,
                                                  char *, long));

extern int FDECL(reverse_jmp_opcode, (int));

struct coord {
    long x;
    long y;
};

struct forloopdef {
    char *varname;
    long jmp_point;
};
static struct forloopdef forloop_list[MAX_NESTED_IFS];
static short n_forloops = 0;


sp_lev *splev = NULL;

static struct opvar *if_list[MAX_NESTED_IFS];

static short n_if_list = 0;

unsigned int max_x_map, max_y_map;
int obj_containment = 0;

int in_container_obj = 0;

/* integer value is possibly an inconstant value (eg. dice notation
   or a variable) */
int is_inconstant_number = 0;

int in_switch_statement = 0;
static struct opvar *switch_check_jump = NULL;
static struct opvar *switch_default_case = NULL;
static struct opvar *switch_case_list[MAX_SWITCH_CASES];
static long switch_case_value[MAX_SWITCH_CASES];
int n_switch_case_list = 0;

int allow_break_statements = 0;
struct lc_breakdef *break_list = NULL;

extern struct lc_vardefs *vardefs; /* variable definitions */


struct lc_vardefs *function_tmp_var_defs = NULL;
extern struct lc_funcdefs *function_definitions;
struct lc_funcdefs *curr_function = NULL;
struct lc_funcdefs_parm * curr_function_param = NULL;
int in_function_definition = 0;
sp_lev *function_splev_backup = NULL;

extern int fatal_error;
extern int got_errors;
extern int line_number;
extern const char *fname;

extern char curr_token[512];


# line 150 "lev_comp.y"
typedef union 
{
    long    i;
    char    *map;
    struct {
        long room;
        long wall;
        long door;
    } corpos;
    struct {
        long area;
        long x1;
        long y1;
        long x2;
        long y2;
    } lregn;
    struct {
        long x;
        long y;
    } crd;
    struct {
        long ter;
        long lit;
    } terr;
    struct {
        long height;
        long width;
    } sze;
    struct {
        long die;
        long num;
    } dice;
    struct {
        long cfunc;
        char *varstr;
    } meth;
} YYSTYPE;
# define CHAR 257
# define INTEGER 258
# define BOOLEAN 259
# define PERCENT 260
# define SPERCENT 261
# define MINUS_INTEGER 262
# define PLUS_INTEGER 263
# define MAZE_GRID_ID 264
# define SOLID_FILL_ID 265
# define MINES_ID 266
# define ROGUELEV_ID 267
# define MESSAGE_ID 268
# define MAZE_ID 269
# define LEVEL_ID 270
# define LEV_INIT_ID 271
# define GEOMETRY_ID 272
# define NOMAP_ID 273
# define OBJECT_ID 274
# define COBJECT_ID 275
# define MONSTER_ID 276
# define TRAP_ID 277
# define DOOR_ID 278
# define DRAWBRIDGE_ID 279
# define object_ID 280
# define monster_ID 281
# define terrain_ID 282
# define MAZEWALK_ID 283
# define WALLIFY_ID 284
# define REGION_ID 285
# define FILLING 286
# define IRREGULAR 287
# define JOINED 288
# define ALTAR_ID 289
# define LADDER_ID 290
# define STAIR_ID 291
# define NON_DIGGABLE_ID 292
# define NON_PASSWALL_ID 293
# define ROOM_ID 294
# define PORTAL_ID 295
# define TELEPRT_ID 296
# define BRANCH_ID 297
# define LEV 298
# define MINERALIZE_ID 299
# define CORRIDOR_ID 300
# define GOLD_ID 301
# define ENGRAVING_ID 302
# define FOUNTAIN_ID 303
# define POOL_ID 304
# define SINK_ID 305
# define NONE 306
# define RAND_CORRIDOR_ID 307
# define DOOR_STATE 308
# define LIGHT_STATE 309
# define CURSE_TYPE 310
# define ENGRAVING_TYPE 311
# define DIRECTION 312
# define RANDOM_TYPE 313
# define RANDOM_TYPE_BRACKET 314
# define A_REGISTER 315
# define ALIGNMENT 316
# define LEFT_OR_RIGHT 317
# define CENTER 318
# define TOP_OR_BOT 319
# define ALTAR_TYPE 320
# define UP_OR_DOWN 321
# define SUBROOM_ID 322
# define NAME_ID 323
# define FLAGS_ID 324
# define FLAG_TYPE 325
# define MON_ATTITUDE 326
# define MON_ALERTNESS 327
# define MON_APPEARANCE 328
# define ROOMDOOR_ID 329
# define IF_ID 330
# define ELSE_ID 331
# define TERRAIN_ID 332
# define HORIZ_OR_VERT 333
# define REPLACE_TERRAIN_ID 334
# define EXIT_ID 335
# define SHUFFLE_ID 336
# define QUANTITY_ID 337
# define BURIED_ID 338
# define LOOP_ID 339
# define FOR_ID 340
# define TO_ID 341
# define SWITCH_ID 342
# define CASE_ID 343
# define BREAK_ID 344
# define DEFAULT_ID 345
# define ERODED_ID 346
# define TRAPPED_STATE 347
# define RECHARGED_ID 348
# define INVIS_ID 349
# define GREASED_ID 350
# define FEMALE_ID 351
# define CANCELLED_ID 352
# define REVIVED_ID 353
# define AVENGE_ID 354
# define FLEEING_ID 355
# define BLINDED_ID 356
# define PARALYZED_ID 357
# define STUNNED_ID 358
# define CONFUSED_ID 359
# define SEENTRAPS_ID 360
# define ALL_ID 361
# define MONTYPE_ID 362
# define GRAVE_ID 363
# define ERODEPROOF_ID 364
# define FUNCTION_ID 365
# define MSG_OUTPUT_TYPE 366
# define COMPARE_TYPE 367
# define UNKNOWN_TYPE 368
# define rect_ID 369
# define fillrect_ID 370
# define line_ID 371
# define randline_ID 372
# define grow_ID 373
# define selection_ID 374
# define flood_ID 375
# define rndcoord_ID 376
# define circle_ID 377
# define ellipse_ID 378
# define filter_ID 379
# define complement_ID 380
# define gradient_ID 381
# define GRADIENT_TYPE 382
# define LIMITED 383
# define HUMIDITY_TYPE 384
# define STRING 385
# define MAP_ID 386
# define NQSTRING 387
# define VARSTRING 388
# define CFUNC 389
# define CFUNC_INT 390
# define CFUNC_STR 391
# define CFUNC_COORD 392
# define CFUNC_REGION 393
# define VARSTRING_INT 394
# define VARSTRING_INT_ARRAY 395
# define VARSTRING_STRING 396
# define VARSTRING_STRING_ARRAY 397
# define VARSTRING_VAR 398
# define VARSTRING_VAR_ARRAY 399
# define VARSTRING_COORD 400
# define VARSTRING_COORD_ARRAY 401
# define VARSTRING_REGION 402
# define VARSTRING_REGION_ARRAY 403
# define VARSTRING_MAPCHAR 404
# define VARSTRING_MAPCHAR_ARRAY 405
# define VARSTRING_MONST 406
# define VARSTRING_MONST_ARRAY 407
# define VARSTRING_OBJ 408
# define VARSTRING_OBJ_ARRAY 409
# define VARSTRING_SEL 410
# define VARSTRING_SEL_ARRAY 411
# define METHOD_INT 412
# define METHOD_INT_ARRAY 413
# define METHOD_STRING 414
# define METHOD_STRING_ARRAY 415
# define METHOD_VAR 416
# define METHOD_VAR_ARRAY 417
# define METHOD_COORD 418
# define METHOD_COORD_ARRAY 419
# define METHOD_REGION 420
# define METHOD_REGION_ARRAY 421
# define METHOD_MAPCHAR 422
# define METHOD_MAPCHAR_ARRAY 423
# define METHOD_MONST 424
# define METHOD_MONST_ARRAY 425
# define METHOD_OBJ 426
# define METHOD_OBJ_ARRAY 427
# define METHOD_SEL 428
# define METHOD_SEL_ARRAY 429
# define DICE 430
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern short yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 2719 "lev_comp.y"


/*lev_comp.y*/
short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
-1, 279,
	123, 200,
	-2, 199,
-1, 393,
	331, 158,
	-2, 157,
-1, 814,
	41, 14,
	-2, 348,
	};
# define YYNPROD 406
# define YYLAST 1186
short yyact[]={

 246, 735, 262, 530, 795,  12, 820, 762, 515, 698,
 272, 243, 242, 301, 710, 730, 274, 280, 281, 131,
 444, 463, 290, 308, 511, 461,  64, 494, 237, 180,
 400, 228, 198, 153, 347, 270, 822, 190, 680, 681,
 453, 295, 181, 310, 682, 286, 518, 387, 191, 192,
 286, 692, 626, 321, 604, 130, 106, 129, 348, 849,
 821, 181, 119, 110, 120, 111, 121, 112, 122, 113,
 123, 114, 124, 115, 125, 116, 126, 117, 127, 118,
 221, 502, 110, 289, 111, 181, 112, 210, 113, 176,
 114, 467, 115, 181, 116, 119, 117, 120, 118, 121,
 732, 122, 733, 123, 596, 124, 186, 125, 206, 126,
   9, 127, 190, 189, 704, 309, 703, 233, 185, 645,
 542, 178, 514, 191, 192, 320, 644, 295, 469, 295,
 517, 516, 276, 277, 858, 197, 205, 531, 467, 218,
 337, 532, 305, 188, 240, 241, 303, 304, 699, 214,
 212, 445, 711, 446, 443, 238, 273, 275, 278, 260,
 239, 495, 662, 264, 235, 487, 271, 488, 823, 495,
 563, 520, 315, 695, 462, 485, 797, 798, 799, 835,
 806, 538, 780, 109, 202,   6,   5, 534, 859, 415,
 831, 286, 202, 332, 244, 330, 336, 302, 772, 683,
 328, 326, 519, 327, 548, 329,  13, 712, 202, 832,
 299, 830, 471, 209, 810, 330, 312, 313, 236, 809,
 328, 326, 322, 327, 305, 329, 773, 221, 303, 304,
 769, 366, 363, 332, 194, 193, 196, 195, 768, 766,
 718, 709, 648, 646, 636, 467, 633, 630, 578, 576,
 378, 379, 365, 364, 382, 562, 559, 455, 294, 307,
 367, 202, 285, 245, 331, 421, 369, 285, 393, 390,
 439, 325, 300, 354, 389, 352, 346, 787, 104, 179,
 316, 318, 642, 331, 567, 553, 441, 440, 756, 754,
 409, 406, 658, 317, 783, 656, 652, 506, 179, 739,
 740, 742, 479, 179, 840, 587, 263, 477, 466, 675,
 220, 179, 475, 473, 296, 707, 505, 450, 287, 211,
 391, 323, 744, 287, 743, 745, 746, 747, 748, 749,
 750, 751, 752, 753, 566, 424, 425, 556, 426, 427,
 428, 429, 430, 288, 294, 433, 294, 408, 405, 793,
 368, 436, 108, 203, 204, 466, 345, 448, 190, 138,
 452, 339, 333, 261, 150, 815, 792, 458, 791, 191,
 192, 790, 789, 657, 786, 468, 655, 651, 690, 689,
 481, 105, 556, 478, 500, 501, 293, 489, 476, 504,
 284, 499, 447, 474, 472, 284, 182, 183, 688, 483,
 296, 687, 296, 460, 686, 522, 521, 498, 285, 271,
 599, 282, 283, 528, 358, 182, 183, 523, 231, 484,
 526, 700, 512, 490, 546, 190, 623, 311, 493, 497,
 621, 622, 184, 513, 182, 183, 191, 192, 190, 182,
 183, 457, 470, 540, 541, 179, 696, 182, 183, 191,
 192, 184, 435, 434, 539, 464, 465, 203, 204, 454,
 535, 227, 466, 225, 287, 203, 204, 226, 224, 222,
 184, 208, 293, 394, 293, 184, 615, 612, 606, 229,
 230, 203, 204, 184, 565, 572, 560, 438, 569, 177,
 575, 609, 583, 584, 571, 291, 292, 291, 292, 549,
 203, 204, 464, 465, 297, 610, 611,  78, 573, 574,
 595, 357, 356, 355, 613, 616, 617, 618, 619, 223,
 529, 386, 219, 174, 173, 172, 620, 171, 199, 543,
 607, 545, 614, 170, 203, 204, 284, 305, 276, 277,
 631, 303, 304, 169, 199, 324, 600, 168, 167, 166,
 165, 208, 200, 201, 164, 624, 163, 282, 283, 639,
 162, 213, 161, 199, 160, 159, 203, 204, 200, 201,
 158, 157, 190, 156, 155, 152, 147, 146, 653, 145,
 144, 182, 183, 191, 192, 151, 143, 200, 201, 229,
 230, 142, 664, 141, 140, 665, 139, 199, 184, 676,
 137, 136, 135, 672, 673, 134, 133, 661, 466, 663,
 132, 128, 496,  11,  10, 668, 338, 184, 640, 736,
 861, 200, 201, 678, 647, 512, 855, 649, 647, 199,
 330, 659, 854, 845, 667, 328, 669, 670, 671, 763,
 329, 330, 826, 808, 807, 705, 328, 326, 863, 327,
 804, 329, 693, 200, 201, 801, 777, 774, 715, 770,
 702, 330, 764, 232, 203, 204, 328, 326, 848, 327,
 720, 329, 330, 719, 713, 716, 708, 328, 721, 837,
 734, 717, 329, 109, 298, 259, 729, 706, 306, 674,
 701, 697, 684, 314, 666, 738, 659, 714, 758, 259,
 654, 759, 760, 761, 757, 755, 767, 650, 330, 649,
 641, 638, 765, 328, 326, 847, 327, 625, 329, 635,
 247, 248, 249, 250, 251, 634, 253, 199, 254, 255,
 252, 257, 256, 779, 778, 259, 632, 629, 628, 510,
 781, 627, 601, 401, 598, 593, 785, 784, 592, 591,
 590, 200, 201, 788, 589, 464, 465, 330, 588, 586,
 582, 258, 328, 326, 603, 327, 581, 329, 580, 579,
 376, 377, 561, 330, 800, 555, 554, 259, 328, 326,
 330, 327, 544, 329, 536, 328, 326, 846, 327, 527,
 329, 525, 524, 492, 814, 491, 482, 825, 480, 459,
 456, 451, 817, 818, 819, 449, 829, 423, 827, 330,
 828, 422, 420, 577, 328, 326, 728, 327, 330, 329,
 419, 418, 839, 328, 326, 844, 327, 417, 329, 558,
 416, 841, 413, 330, 412, 411, 552, 812, 328, 326,
 813, 327, 404, 329, 850, 852, 402, 851, 330, 853,
 399, 398, 108, 328, 326, 856, 327, 397, 329, 862,
  58, 374, 864,  59,  85,  84,  88,  89,  87, 102,
  68,  69, 373, 372, 551,  86, 103,  94, 371, 370,
 353,  60,  83, 100,  66,  90,  95,  92, 101,  62,
 350,  71,  63,  73,  70,  72,  91,  97, 349,  93,
 343, 330, 567, 346, 550, 726, 328, 326, 727, 327,
 330, 329, 342, 341,  96, 328, 326, 388, 327, 340,
 329,  67,  77, 334,  98, 187,  99,  79,  65, 843,
 834,  76, 107, 330,  74, 833,  80, 432, 328, 326,
 330, 327, 811, 329, 233, 328, 326, 805, 782, 802,
 329, 776, 775, 346, 771,  61, 330,  81, 203, 204,
 385, 328, 326, 725, 327, 724, 329, 723, 722, 691,
 685, 495, 203, 204, 647, 597, 585, 557, 503,  82,
 106, 431, 396, 384, 383, 381, 119, 110, 120, 111,
 121, 112, 122, 113, 123, 114, 124, 115, 125, 116,
 126, 117, 127, 118, 380, 351, 344, 267, 203, 204,
 211, 154, 375, 794, 247, 248, 249, 250, 251, 608,
 253, 199, 254, 255, 252, 257, 256, 410, 247, 248,
 249, 250, 251, 403, 253, 199, 254, 255, 252, 257,
 256, 279, 842,   2, 803, 200, 201,   7, 508, 392,
 203, 204, 265, 266, 507, 258, 149,  75, 547, 200,
 201, 816, 838, 731, 247, 248, 249, 250, 251, 258,
 253, 199, 254, 255, 252, 257, 256, 594, 148, 677,
 509, 395,  57,  56,  55,  54,  53,  52,  51,  50,
  49,  48,  47,  46,  45, 200, 201,  44,  43,  42,
  41,  40,  39,  38,  37, 258, 247, 248, 249, 250,
 251,  36, 253, 199, 254, 255, 252, 257, 256,  35,
  34,  33,  32,  31,  30,  29,  28,  27,  26,  25,
  24,  23,  22,  21,  20,  19,  18, 200, 201,  17,
  16,  14,  15,   8,   3,   1, 437, 258, 694, 537,
 533, 207, 269, 268, 175,   4, 335, 836, 860, 215,
 216, 217, 679, 414, 568, 564, 361, 360, 362, 359,
 570, 796, 486, 824, 737, 602, 605, 407, 637, 234,
 857, 660, 741, 643, 442, 319 };
short yypact[]={

 -84,-1000,-1000, -84,-214, 556, 555,-1000, 592, 553,
-328,-330,-1000, 592,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000, 552, 548,
 547, 544, 543, 542, 298, 538, 536, 535, 533, 528,
 522, 521, 519, 518,-1000,-1000, 273,  92, 517,-1000,
-1000,-354, 971, 516,-1000, 515, 513, 512, 507, 506,
 504, 502, 498, 496, 492, 491, 490, 489, 485, 475,
 469, 467, 466, 465,-1000,-1000,-1000,-299,-1000,  45,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-219,-1000,
 881,-1000,-348, -30, 152, 152, 173, 970, 187,-313,
  77, -95,-153, 152, 152,  45, 737,  45, 272, 183,
  45,-1000,-1000, 967,  53, 152, 221, 152, 151,  87,
  87,  77, 737, 173,-116,  77,-270,-270, 737, 737,
  77, 253, 173,-260, 737, 260,-1000,-1000, 178,-1000,
-1000,  21,-1000, 271,-1000,-1000, 879,-117, 570,-1000,
-1000,-1000, 270, 875, 869,-1000, 868, 856,-1000, 966,
-1000, 265,  18,-1000,-326, 854, 846,-1000, 965,  17,
 836,  15, 919, 455, 570, 454, 453, 356,-1000,-1000,
  40,   2,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
 259,-1000,-1000,   8, 835,-1000,-1000, 834,-1000,-1000,
 829, 828, 817, 919,-1000, 974,-1000,  77,  77, 152,
 152, 964, 945, 152, 944, 943, 920, 737,-1000, 695,
 873,  45,-1000, 592, 227, 183, 592,-1000, 941, 813,
-1000, 919, 570, 807, 806, 699,-1000,-1000, 802,-1000,
 798,-1000,-1000, 257,-1000,-1000,  34,-1000,-1000,-1000,
-1000,-1000, 256,-1000,-1000,  33,-1000,-1000,-1000,-1000,
 791,-1000,-1000,-1000,-1000,-1000, 790, 788, -71,-1000,
-1000, 786,-1000, 783, 777, 776, 768,   7, 767, 763,
-1000,-1000,-1000,  45,  45,-1000,  45,  45,  45,  45,
  45, 940, 896,  45,-219,-1000,-1000,-1000,-348,  45,
 230,  30,  29,-162, 737,  45, 761, 224, 757,-273,
 173,  -1, 756, 279, 755, 737,  51,   5,  89, 269,
 268, 263, 258, 919,-1000,-1000, 570, 754,  45, 752,
-153, 737,-137,-146,  45, 737,-1000,-1000, 751, 749,
 659, 351,-1000, 152, 152,-301,-1000, 937, 152, 223,
 172,-1000,-1000,-1000,-1000,-332,-1000,  53,-199,-188,
-340, -57,-141, 183, 152,  45, 748, 747,  45, 745,
 183, 173,-172, 147, 740,-1000, 141,  98,  98,-201,
 173, 738, 173, 152, 158, 811, 593, 593,-1000,-1000,
-1000,-1000,-1000, 781,-1000,-1000, 743,-1000,-1000,  28,
-1000, 732, 731,-1000,-1000,-1000, 246, 936, 736,  -2,
-1000,-326, 570,-1000,-1000, 728,  -3,-1000,-1000,-142,
-1000,-1000, 205,-1000,-1000, 243,-1000,  27,-1000,  10,
-1000,   1,-1000,  45,-1000, 144,-1000, 904,-1000,-348,
  -9, 720, -10, 725,-1000, 724, 722,-1000,-1000, 716,
-1000, 152, 152, 935, 715, 181, 714, 710, 706, 645,
 705, 704, 701,-1000,-1000,-1000,-1000, 183,-227, 934,
 700,-1000, 352,-1000,-1000, 699,-1000,-1000,-1000,-1000,
 698,-1000,-1000, 671,-331, 168, 624,-333,-1000, 697,
 694,-1000,-1000, 693, -11,-1000,-172, 692, -12,-1000,
-1000, 681,-1000, 675, -14, 667,-1000,  45, 572,-1000,
-1000,-1000,-1000, 666,  25,-194, -15,-1000,-1000, 933,
-1000, -16, 665, 663, 252,-1000,  45, 656, 251,-1000,
 248,-1000, 919,-1000,-1000, 570, 587,-1000, 652,-151,
-153,-348,  45,-1000, 650,-1000, 737,-143, 737, 737,
 737,  45,  45, 649, 186,-1000, 183,-1000,-332,-352,
-342, -60, 648,-1000, 929,-1000,-1000, 346,-1000, 343,
 340,-1000,-1000, 321,-1000,-1000,-1000, 320,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000, 928,-334,-270, 133,
 647,-1000, 108, 646,  98,-205, 583,-1000,-207, 919,
-1000,-172, 643,-1000,-1000,-1000, 222,-1000, 632, -17,
-106,-1000, 205, 604,-172,-1000,  10,-1000,   1, -18,
 629,-1000,-1000,-1000, 570, 626,  45, 927,-1000, 926,
 924, 922, 864, 772,  45,-243,-1000, 183,-1000,-1000,
-1000,-1000,-1000, 575, -27,-1000, 151,-348,  45,  45,
  45,-1000,-1000, 595, 618,-185,-1000, -19, 595, -20,
-1000, -28, 615,-1000,-1000, 913, -61,-1000, -32, 613,
 911,-1000,-1000,-1000,-1000, 910,-1000,-1000, 612,-106,
  45, 919,-1000,-1000,-1000,-1000,-1000,-104,  45, 903,
 169,-243, -34, 316,-1000,-1000,  20,-1000, 570,-1000,
-1000,-1000,-348,-1000,-1000,-1000,-1000,-1000, 314, 313,
 310,-1000,-1000, 308,-1000,-1000, 291,-1000, 570,-1000,
-1000,-1000,-1000,-110, 108, 611, 908,-1000, 606, 906,
 -81,-1000, 600, 599, -39,-1000,-1000, -44,-1000,-1000,
 901, 796,  45,-1000,-1000, 307,-1000,-1000, 570,  45,
  45,  45,-325,-145, 183,-1000, 598,-1000,-1000,-1000,
 595,-188,-1000, 183, -47,-1000,-1000, -69, -49, 894,
 889,-1000,-1000,-107, 635,-1000, 592,-1000,-1000,-1000,
-1000, 180,-1000,-1000,-1000,-1000,-110,-1000, 888,-1000,
 784, 589, 746,-1000,-1000, 674, 627,-324, 592,-1000,
-325,-1000, 183,-1000,-1000,-172,-1000,-1000, 588,-1000,
-1000,-1000,-1000, 582, 152,-125, 576, 575,-1000,-1000,
 607, 152,-1000,-1000,-1000 };
short yypgo[]={

   0,  16,   8,1185,  23,  28,   3,1184,1183,  20,
  30,  14,1182,1181,1180,1179,  27,1178,1177,1176,
1175,1174,   5,   2,1173,1172, 118,1171,   4,   7,
  34, 507,  32,  31,  21,   6,  18,  22,1170,1169,
1168,1167,1166,1165,1164,1163,   1,  13,  47, 263,
1162,1161,1160,1159,1158,1157,1156,1155, 381, 278,
  26,1154,1153,1152,  35,  87,1151, 108,1150,1149,
1148,   9,1146,1145,1043,1144,1143,1142,   0, 206,
1141,1140,1139,1136,1135,1134,1133,1132,1131,1130,
1129,1128,1127,1126,1125,1124,1123,1122,1121,1120,
1119,1111,1104,1103,1102,1101,1100,1099,1098,1097,
1094,1093,1092,1091,1090,1089,1088,1087,1086,1085,
1084,1083,1082,  11,  10,  25,  17,  83, 522,1081,
1080,1079,1078,  12,1077,  15,1063,1062,1061,1058,
1057,1056,1054,1053,1052,1049,1048, 259,1044,1042,
1041,1033, 343,1027,1019,1013, 113,  29,  24, 739 };
short yyr1[]={

   0,  73,  73,  74,  74,  75,  57,  57,  56,  56,
  77,  77,  77,  77,  55,  55,  54,  54,  46,  46,
  14,  14,  76,  76,  26,  26,  22,  22,  23,  79,
  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
  79,  79,  79,  59,  59,  59,  59,  59,  59,  59,
  59,  59,  58,  58,  58,  58,  58,  58,  58,  58,
  58,  60,  60,  60,  61,  61,  86,  85,  85,  85,
  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
  85,  85,  38,  38,  44,  44,  43,  43,  42,  42,
  41,  41,  39,  39,  40,  40, 129, 131, 101, 102,
  99,  45,  45,  31,  31,  31, 132, 134,  94, 135,
 135, 137, 136, 138, 136, 100, 139, 139, 140, 141,
  95, 142,  96, 143,  98, 144,  97, 145, 146, 145,
  80, 111, 111, 111,  84,  84,  65, 147, 148, 114,
 149, 113,  10,  10,  68,  68,  69,  69,  70,  70,
  71,  71,  88,  88,  15,  15,  13,  13,  16,  16,
  11,  11, 104, 104, 104,   1,   1,   2,   2, 106,
 151, 106, 150,  20,  20,  21,  21,  21,  21,  21,
  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,
  21,  35,  35,  35, 107, 153, 107, 152,  18,  18,
  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
  19,  19,  19,  19,  19, 121,  89, 105, 105, 122,
 122, 103, 118, 119, 110, 120,  83,  17,  17,  92,
 115, 109,  72,  72, 117, 116,  87, 108, 155, 112,
  24,  24,  81,  82,  82,  82,  93,  90,  91,  91,
   3,   3,   4,   4,  29,  29,  28,  28,  27,  27,
  27,   5,   5,   6,   6,   7,   7,   7,  12,  12,
  12,   8,   8,   9, 156, 156, 156, 133,  78,  78,
  78,  78,  32,  32,  32,  30,  30, 128, 128, 128,
  33, 125, 125, 125,  34,  34, 126, 126, 126,  36,
  36,  36,  36, 127, 127, 127,  37,  37,  37,  37,
 124, 124, 123, 123, 123, 123, 123, 123, 123, 123,
 123, 123, 123,  50,  50, 158, 159, 159, 130, 130,
  64,  64,  63,  63,  62,  62,  49,  49,  49,  49,
  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
  49,  49,  49,  49,  49,  48,  48, 157,  47,  47,
  47, 154, 154, 154, 154,  51,  51,  52,  52,  53,
  53,  25,  25,  67,  67,  66 };
short yyr2[]={

   0,   0,   1,   1,   2,   3,   3,   5,   1,   1,
   5,   5,   3,  16,   0,   2,   0,   2,   0,   2,
   1,   1,   0,   3,   3,   1,   0,   2,   3,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   3,   3,   5,   3,
   5,   5,   5,   3,   3,   5,   5,   5,   7,   7,
   7,   5,   1,   3,   1,   3,   1,   3,   1,   3,
   1,   3,   1,   3,   1,   3,   0,   0,   8,   4,
   1,   0,   1,   1,   5,   3,   0,   0,   9,   0,
   2,   0,   5,   0,   4,   1,   2,   1,   6,   0,
   3,   0,   6,   0,   4,   0,   4,   1,   0,   4,
   3,   1,   3,   3,   5,   5,   7,   4,   0,  10,
   0,  12,   0,   2,   5,   1,   5,   1,   5,   1,
   5,   1,   9,   5,   1,   1,   1,   1,   1,   3,
   1,   1,   1,   7,   5,   1,   1,   1,   1,   3,
   0,   5,   4,   0,   3,   1,   1,   1,   1,   2,
   1,   1,   1,   1,   1,   3,   3,   3,   1,   1,
   3,   1,   1,   3,   3,   0,   5,   2,   0,   3,
   1,   3,   1,   3,   3,   1,   1,   3,   1,   1,
   1,   3,   1,   1,   1,   5,   7,   5,   8,   1,
   3,   5,   5,   7,   7,   6,   5,   0,   2,   3,
   3,   3,   1,   5,   9,   5,   3,   3,   0,  10,
   0,   1,   7,   5,   5,   3,   5,   7,   9,   1,
   1,   1,   1,   1,   0,   2,   1,   3,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   3,   1,   1,   4,   1,   1,   4,   1,   1,   4,
   1,   4,   5,   1,   3,   1,   3,   1,   1,   4,
   9,   1,   1,   4,   1,   5,   1,   1,   4,   1,
   1,   5,   1,   1,   1,   4,   1,   1,   5,   1,
   1,   3,   1,   1,   3,   1,   4,   3,   3,   3,
   3,   3,   3,   1,   1,   3,   1,   3,   0,   1,
   1,   1,   1,   3,   0,   1,   1,   2,   2,   4,
   6,   4,   6,   6,   6,   6,   2,   6,   8,   8,
  10,  14,   2,   1,   3,   1,   3,   1,   1,   1,
   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
   1,   1,   1,   1,  10,   9 };
short yychk[]={

-1000, -73, -74, -75, -57, 270, 269, -74, -76, 324,
  58,  58, -22, -79, -80, -77, -81, -82, -83, -84,
 -85, -86, -87, -88, -89, -90, -91, -92, -93, -94,
 -95, -96, -97, -98, -99,-100,-101,-102,-103,-104,
-105,-106,-107,-108,-109,-110,-111,-112,-113,-114,
-115,-116,-117,-118,-119,-120,-121,-122, 268, 271,
 289, 363, 297, 300, -60, 336, 292, 329, 278, 279,
 302, 299, 303, 301, 342,-140, 339, 330, -31, 335,
 344, 365, 387, 290, 273, 272, 283, 276, 274, 275,
 293, 304, 295, 307, 285, 294, 322, 305, 332, 334,
 291, 296, 277, 284, -59, -58, 388, 340, 260,  91,
 395, 397, 399, 401, 403, 405, 407, 409, 411, 394,
 396, 398, 400, 402, 404, 406, 408, 410,  58, 385,
 385, -22,  58,  58,  58,  58,  58,  58,  61,  58,
  58,  58,  58,  58,  58,  58,  58,  58,-132,-141,
  91, -31,  58, 387,  40,  58,  58,  58,  58,  58,
  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
  58,  58,  58,  58,  58, -61, 388, -58,-123, 258,
-157,  40, 394, 395, 430, -26, 325,  44,-124,-156,
 385, 396, 397, 265, 264, 267, 266, -78, -32, 376,
 400, 401,  40, 313, 314, -78, -67, -66, 298,  40,
 -65,  40,-123, 374,-124, -53, -52, -51, -78,-128,
 123,  40, 282, 332, 281, 276, 280, 274, -33, 402,
 403, -59,-128,  40, -15, 259, 313,  -5, 308, 313,
 -78, -78,-133,-123, -48, -49, -78, 369, 370, 371,
 372, 373, 379, 375, 377, 378, 381, 380, 410,  40,
-123,  91, -23, 123,-133,-144,-143,  40, -62, -63,
 -64,-123,-124, -78,  -1, -78, 317, 318, -78,-150,
-126, -36, 406, 407, 385, 257,  40, 313,-152,-127,
 -37, 408, 409, 385, 257,  40, 313,-152,-128, -48,
 -67, -47, 313, 262, 263, 258,-128,-147,  -4, 385,
 313,-147, -48, -48,-128, -78, -67,  40, -67,  -3,
 385, 313, -48,  61, 367,  93,  43,  45,  42,  47,
  37, 262,-123,  91,  44, -56, 313, 257,  46,  91,
  44,  44,  44,  44,  40,  91, 258, -30, 384,  44,
  44,  40, 258,  44, 258,  58,  58,  58,  58, -39,
 -41, -42, -40,-123, -32, -33,-124, 258,  91, 258,
  44,  44,  44,  44,  44,  38,-128,-128, -78, -78,
  40,  40, -78,  40,  40,  40, -49, -48,  44,-133,
 -22,  93,-145, -23, -79,-129,  41,  44,  44,  44,
 -10,  44,  44,-151,  44,  91, 257, -18,  91, 257,
-153,  44,  44,  44, -45, 260,  44,  44,  44,  44,
  44, 258,  44,  44,-123,-123,-123,-123,-123,-123,
-123,  41,  41,-123, -26,-156,-123, -72, 257,  40,
 257, 257,  -7, 316,  -9, 313, 315, -48,-123,  44,
  93,  44,-124, 313, -67, 258,  44, -65, -47,  44,
 -48,-125, 123, -34, 404, 405, 257,  40,-126, 123,
-127, 123, 125,  44, 125,  44, 125,  44, 125,  44,
  44,-123,  44,  -5, -48, 312, -25, 311, 313,-133,
 -48,  44,  44, -48, -16, 312, 261, -48,-125,  40,
 -78, -78, 382,  41, -78,  93, 125,-142,-146,-130,
-159,-158, -60, -64, 321,  -2, 319, 318, 386, 259,
 312, -23, -78,-123,  44,  44,-123,  44, -23, -67,
  -6, 309, 313, -68,  40, 313,  44, -69,  40, 313,
-125,-125, 321, -67,  44, -67, -78,-139,  46, 341,
  93,  93,  93, 257,  44,  44,  91,  41,  93, 258,
 -30,  44, 258, 312, -43, -34,  91, 257, -44, -36,
 -38, -37,-123, -32, -33,-124, 258,  93, 258,  44,
  44,  44,  44, -78, -78,  41,  44, 124,  44,  44,
  44,  44,  44,  44,-134, -23, 331,  41,  44,  58,
 -10,  44, -20,  93, 385, -19, 310, 362,-154, 323,
 337, 338, 309, 346, 364, 308, 347, 348, 349, 350,
 -78, 262, 263, 258,-157,  93, 385,  44,  44,  44,
 258,  -6,  44, 258,  44,  44, 258, -17,  44,-123,
  46,  44, 257,  -8, 320, 313, 258,  41, 258,  44,
  44, 125,  44,-123,  44, 125,  44, 125,  44,  44,
 -13, -16, 313,  -5,-124,-133,  44, -48, -16, -48,
 -48, -48,-123,-123,  40, 123, -23,-131,-158, -50,
 390, 391, 386, 259,  44,  41,  58,  58,  58,  58,
  58,  41, 385,  -4, -70,  40, 313,  44, -71,  40,
 313,  44,-125, 321, 321,  -6,  44,  93,  44, 258,
 -11, 258, 313, -34,  93,  -6, -36, -37, 258,  44,
  44,-123,  41,  41,  41,  41,  41,  44,  44,-123,
-135,-136, 343, 345, -23, -46,  44, -21,-124, 326,
 327, -12, 328, 351, 349, 352, 353, 354, 355, 356,
 357, 358, 359, 360, 316,  -9, 315,-126,-124,-133,
-133,-133, -29,  44,  44,  -1, 258, -29, 258, 258,
  44,  41, 259, 258,  44,  41,  41,  44, -11,-133,
 286,-123,  45, 125,-135, -47,  58, 257,-124,  58,
  58,  58,  58,  58,-155, -28, -27, 286, 287, 288,
 -71,  44,  41,-148,  44,  41, 261,  44,  44, 258,
 258,  41,  41,  44,-123,  58,-138,-133,-133,-133,
 -35, 385, 361, 313, -24, -23,  44, -29,  -2, -23,
 258, 259, 258,  41,  41, 286, -55,  44,-137, -22,
 124, -28,-149,  41,  41,  44,  41,  41,  41, 383,
 -22, -35, -23,  -6,  44,  44, -78, -14, 259, 313,
 -54,  44, -46,  41, -78 };
short yydef[]={

   1,  -2,   2,   3,  22,   0,   0,   4,  26,   0,
   0,   0,   5,  26,  29,  30,  31,  32,  33,  34,
  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
  65,  66,  67,  68,  69,  70,  71,  72,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0, 279,   0,   0, 136, 149,   0,   0,   0, 130,
 145,   0,   0,   0, 192,   0,   0,   0,   0,   0,
   0,   0,   0, 161,   0,   0,   0,   0,   0,   0,
   0,   0,   0, 249,  91,  92,  93,   0, 133,   0,
  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
  83,  84,  85,  86,  87,  88,  89,  90,   0,   6,
   0,  27,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0, 155, 153,   0, 364,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,  94,  95,   0, 342,
 343,   0, 345,   0, 387,  23,  25,   0, 160, 340,
 304, 305,   0,   0,   0,  12,   0,   0, 308,   0,
 310,   0,   0, 313,   0, 275,   0, 403,   0,   0,
   0,   0,  97,   0,  99,   0,   0,   0, 103, 104,
   0,   0, 399, 400, 397, 398, 395, 396, 317, 318,
   0,  96, 266,   0,   0, 184, 185,   0, 291, 292,
   0,   0,   0, 307, 259, 385, 366,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0, 383,   0,
   0,   0, 150,  26,   0,   0,   0, 126,   0, 365,
 362, 360, 361,   0,   0, 172, 195, 196,   0,  -2,
   0, 326, 327,   0, 329, 330,   0, 332, 224, 228,
 333, 334,   0, 336, 337,   0, 339, 225, 267, 261,
   0, 162, 163, 388, 389, 390,   0,   0, 131, 282,
 283,   0, 260,   0,   0,   0,   0,   0,   0,   0,
 280, 281, 250,   0,   0, 135,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   7,   8,   9,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0, 315,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0, 122, 120, 118, 124, 342,   0,   0,
   0,   0,   0,   0,   0,   0, 367, 368,   0,   0,
   0,   0, 376,   0,   0,   0, 382,   0,   0,   0,
   0, 151, 156,  -2, 154, 358, 129,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0, 227,   0,   0,
   0,   0,   0,   0,   0, 132,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0, 347, 348, 349, 350,
 351, 344, 352,   0,  24, 341,   0,  10, 262,   0,
  11,   0,   0, 295, 296, 297,   0,   0,   0,   0,
 314,   0, 273, 274, 256,   0,   0, 164, 165,   0,
  98, 100,   0, 321, 322,   0, 324,   0, 101,   0,
 102,   0, 105,   0, 106,   0, 107,   0, 111,   0,
   0,   0,   0,   0, 183,   0,   0, 401, 402,   0,
 386,   0,   0,   0,   0, 188,   0,   0,   0,   0,
   0,   0,   0, 384, 276, 137,  28,   0,   0,   0,
 359, 356,   0, 363, 251, 172, 197, 198, 194, 173,
 247, 201, 203,   0,   0,   0,   0,   0, 226,   0,
   0, 293, 294,   0,   0, 175,   0,   0,   0, 177,
 265,   0, 252,   0,   0, 257, 245,   0,   0, 147,
 134, 346, 306,   0,   0,   0,   0, 309, 311,   0,
 316,   0,   0,   0,   0, 116,   0,   0,   0, 114,
   0, 112, 123, 121, 119, 125,   0, 319,   0,   0,
   0,   0,   0, 369,   0, 371,   0,   0,   0,   0,
   0,   0,   0,   0,   0, 152,   0, 127,   0,   0,
   0,   0, 202, 328,   0, 229, 230,   0, 232,   0,
   0, 235, 236,   0, 238, 239, 240,   0, 242, 243,
 244, 391, 392, 393, 394, 335,   0,   0,   0,   0,
   0, 167,   0,   0,   0,   0,   0, 255,   0, 148,
 146,   0,   0, 272, 301, 302,   0, 312,   0,   0,
   0, 108,   0,   0,   0, 109,   0, 110,   0,   0,
   0, 186, 187, 246, 277,   0,   0,   0, 189,   0,
   0,   0,   0,   0,   0, 139, 159,   0, 357, 355,
 353, 354, 193,  18,   0, 331,   0,   0,   0,   0,
   0, 338, 254, 284,   0,   0, 179,   0, 284,   0,
 181,   0,   0, 253, 258,   0,   0, 303,   0,   0,
   0, 190, 191, 117, 323,   0, 115, 113,   0,   0,
   0, 370, 372, 373, 374, 375, 377,   0,   0,   0,
   0, 139,   0,   0, 128, 248,   0, 204, 205, 206,
 207, 208,   0, 210, 211, 212, 213, 214,   0,   0,
   0, 218, 219,   0, 298, 299,   0, 231, 233, 234,
 237, 241, 268,   0,   0,   0,   0, 168,   0,   0,
   0, 263,   0,   0,   0, 166, 325,   0, 182, 278,
   0,   0,   0, 138, 140,   0, 143,  19, 209,   0,
   0,   0,   0,   0, 270, 285, 286, 288, 289, 290,
 284,   0, 174,   0,   0, 176, 264,   0,   0,   0,
   0, 378, 379,   0,  -2, 141,  26, 215, 216, 217,
 220, 221, 222, 300, 269, 271,   0, 170,   0, 169,
   0,   0,   0, 405, 320,   0,   0,   0,  26, 144,
   0, 287,   0, 178, 180,   0, 404, 380,   0,  15,
 142, 223, 171,   0,   0,   0,  16,  18,  20,  21,
   0,   0,  13, 381,  17 };
#ifndef lint
static char yaccpar_sccsid[] = "@(#)yaccpar	4.1	(Berkeley)	2/11/83";
#endif

#
# define YYFLAG -1000
# define YYERROR goto yyerrlab
# define YYACCEPT return(0)
# define YYABORT return(1)

/*	parser for yacc output	*/

#ifdef YYDEBUG
int yydebug = 0; /* 1 for debugging */
#endif
YYSTYPE yyv[YYMAXDEPTH]; /* where the values are stored */
int yychar = -1; /* current input token number */
int yynerrs = 0;  /* number of errors */
short yyerrflag = 0;  /* error recovery flag */

yyparse() {

	short yys[YYMAXDEPTH];
	short yyj, yym;
	register YYSTYPE *yypvt;
	register short yystate, *yyps, yyn;
	register YYSTYPE *yypv;
	register short *yyxi;

	yystate = 0;
	yychar = -1;
	yynerrs = 0;
	yyerrflag = 0;
	yyps= &yys[-1];
	yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

#ifdef YYDEBUG
	if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
#endif
		if( ++yyps> &yys[YYMAXDEPTH] ) { yyerror( "yacc stack overflow" ); return(1); }
		*yyps = yystate;
		++yypv;
		*yypv = yyval;

 yynewstate:

	yyn = yypact[yystate];

	if( yyn<= YYFLAG ) goto yydefault; /* simple state */

	if( yychar<0 ) if( (yychar=yylex())<0 ) yychar=0;
	if( (yyn += yychar)<0 || yyn >= YYLAST ) goto yydefault;

	if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
		yychar = -1;
		yyval = yylval;
		yystate = yyn;
		if( yyerrflag > 0 ) --yyerrflag;
		goto yystack;
		}

 yydefault:
	/* default state action */

	if( (yyn=yydef[yystate]) == -2 ) {
		if( yychar<0 ) if( (yychar=yylex())<0 ) yychar = 0;
		/* look through exception table */

		for( yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=yystate) ; yyxi += 2 ) ; /* VOID */

		while( *(yyxi+=2) >= 0 ){
			if( *yyxi == yychar ) break;
			}
		if( (yyn = yyxi[1]) < 0 ) return(0);   /* accept */
		}

	if( yyn == 0 ){ /* error */
		/* error ... attempt to resume parsing */

		switch( yyerrflag ){

		case 0:   /* brand new error */

			yyerror( "syntax error" );
		yyerrlab:
			++yynerrs;

		case 1:
		case 2: /* incompletely recovered error ... try again */

			yyerrflag = 3;

			/* find a state where "error" is a legal shift action */

			while ( yyps >= yys ) {
			   yyn = yypact[*yyps] + YYERRCODE;
			   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
			      yystate = yyact[yyn];  /* simulate a shift of "error" */
			      goto yystack;
			      }
			   yyn = yypact[*yyps];

			   /* the current yyps has no shift onn "error", pop stack */

#ifdef YYDEBUG
			   if( yydebug ) printf( "error recovery pops state %d, uncovers %d\n", *yyps, yyps[-1] );
#endif
			   --yyps;
			   --yypv;
			   }

			/* there is no state on the stack with an error shift ... abort */

	yyabort:
			return(1);


		case 3:  /* no shift yet; clobber input char */

#ifdef YYDEBUG
			if( yydebug ) printf( "error recovery discards char %d\n", yychar );
#endif

			if( yychar == 0 ) goto yyabort; /* don't discard EOF, quit */
			yychar = -1;
			goto yynewstate;   /* try again in the same state */

			}

		}

	/* reduction by production yyn */

#ifdef YYDEBUG
		if( yydebug ) printf("reduce %d\n",yyn);
#endif
		yyps -= yyr2[yyn];
		yypvt = yypv;
		yypv -= yyr2[yyn];
		yyval = yypv[1];
		yym=yyn;
			/* consult goto table to find next state */
		yyn = yyr1[yyn];
		yyj = yypgo[yyn] + *yyps + 1;
		if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn ) yystate = yyact[yypgo[yyn]];
		switch(yym){
			
case 5:
# line 290 "lev_comp.y"
{
			if (fatal_error > 0) {
				(void) fprintf(stderr,
              "%s: %d errors detected for level \"%s\". No output created!\n",
					       fname, fatal_error, yypvt[-2].map);
				fatal_error = 0;
				got_errors++;
			} else if (!got_errors) {
				if (!write_level_file(yypvt[-2].map, splev)) {
                                    lc_error("Can't write output file for '%s'!",
                                             yypvt[-2].map);
				    exit(EXIT_FAILURE);
				}
			}
			Free(yypvt[-2].map);
			Free(splev);
			splev = NULL;
			vardef_free_all(vardefs);
			vardefs = NULL;
		  } break;
case 6:
# line 313 "lev_comp.y"
{
		      start_level_def(&splev, yypvt[-0].map);
		      yyval.map = yypvt[-0].map;
		  } break;
case 7:
# line 318 "lev_comp.y"
{
		      start_level_def(&splev, yypvt[-2].map);
		      if (yypvt[-0].i == -1) {
			  add_opvars(splev, "iiiiiiiio",
				     VA_PASS9(LVLINIT_MAZEGRID, HWALL, 0,0,
					      0,0,0,0, SPO_INITLEVEL));
		      } else {
			  int bg = (int) what_map_char((char) yypvt[-0].i);

			  add_opvars(splev, "iiiiiiiio",
				     VA_PASS9(LVLINIT_SOLIDFILL, bg, 0,0,
					      0,0,0,0, SPO_INITLEVEL));
		      }
		      add_opvars(splev, "io",
				 VA_PASS2(MAZELEVEL, SPO_LEVEL_FLAGS));
		      max_x_map = COLNO-1;
		      max_y_map = ROWNO;
		      yyval.map = yypvt[-2].map;
		  } break;
case 8:
# line 340 "lev_comp.y"
{
		      yyval.i = -1;
		  } break;
case 9:
# line 344 "lev_comp.y"
{
		      yyval.i = what_map_char((char) yypvt[-0].i);
		  } break;
case 10:
# line 350 "lev_comp.y"
{
		      int filling = (int) yypvt[-0].terr.ter;

		      if (filling == INVALID_TYPE || filling >= MAX_TYPE)
			  lc_error("INIT_MAP: Invalid fill char type.");
		      add_opvars(splev, "iiiiiiiio",
				 VA_PASS9(LVLINIT_SOLIDFILL, filling,
                                          0, (int) yypvt[-0].terr.lit,
                                          0,0,0,0, SPO_INITLEVEL));
		      max_x_map = COLNO-1;
		      max_y_map = ROWNO;
		  } break;
case 11:
# line 363 "lev_comp.y"
{
		      int filling = (int) what_map_char((char) yypvt[-0].i);

		      if (filling == INVALID_TYPE || filling >= MAX_TYPE)
			  lc_error("INIT_MAP: Invalid fill char type.");
                      add_opvars(splev, "iiiiiiiio",
				 VA_PASS9(LVLINIT_MAZEGRID, filling, 0,0,
					  0,0,0,0, SPO_INITLEVEL));
		      max_x_map = COLNO-1;
		      max_y_map = ROWNO;
		  } break;
case 12:
# line 375 "lev_comp.y"
{
		      add_opvars(splev, "iiiiiiiio",
				 VA_PASS9(LVLINIT_ROGUE,0,0,0,
					  0,0,0,0, SPO_INITLEVEL));
		  } break;
case 13:
# line 381 "lev_comp.y"
{
                      int fg = (int) what_map_char((char) yypvt[-11].i),
                          bg = (int) what_map_char((char) yypvt[-9].i);
                      int smoothed = (int) yypvt[-7].i,
                          joined = (int) yypvt[-5].i,
                          lit = (int) yypvt[-3].i,
                          walled = (int) yypvt[-1].i,
                          filling = (int) yypvt[-0].i;

		      if (fg == INVALID_TYPE || fg >= MAX_TYPE)
			  lc_error("INIT_MAP: Invalid foreground type.");
		      if (bg == INVALID_TYPE || bg >= MAX_TYPE)
			  lc_error("INIT_MAP: Invalid background type.");
		      if (joined && fg != CORR && fg != ROOM)
			  lc_error("INIT_MAP: Invalid foreground type for joined map.");

		      if (filling == INVALID_TYPE)
			  lc_error("INIT_MAP: Invalid fill char type.");

		      add_opvars(splev, "iiiiiiiio",
				 VA_PASS9(LVLINIT_MINES, filling, walled, lit,
					  joined, smoothed, bg, fg,
					  SPO_INITLEVEL));
			max_x_map = COLNO-1;
			max_y_map = ROWNO;
		  } break;
case 14:
# line 410 "lev_comp.y"
{
		      yyval.i = 0;
		  } break;
case 15:
# line 414 "lev_comp.y"
{
		      yyval.i = yypvt[-0].i;
		  } break;
case 16:
# line 420 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_COPY));
		      yyval.i = 0;
		  } break;
case 17:
# line 425 "lev_comp.y"
{
		      yyval.i = 1;
		  } break;
case 18:
# line 431 "lev_comp.y"
{
		      yyval.i = -1;
		  } break;
case 19:
# line 435 "lev_comp.y"
{
		      yyval.i = what_map_char((char) yypvt[-0].i);
		  } break;
case 22:
# line 446 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(0, SPO_LEVEL_FLAGS));
		  } break;
case 23:
# line 450 "lev_comp.y"
{
		      add_opvars(splev, "io",
                                 VA_PASS2((int) yypvt[-0].i, SPO_LEVEL_FLAGS));
		  } break;
case 24:
# line 457 "lev_comp.y"
{
		      yyval.i = (yypvt[-2].i | yypvt[-0].i);
		  } break;
case 25:
# line 461 "lev_comp.y"
{
		      yyval.i = yypvt[-0].i;
		  } break;
case 26:
# line 467 "lev_comp.y"
{
		      yyval.i = 0;
		  } break;
case 27:
# line 471 "lev_comp.y"
{
		      yyval.i = 1 + yypvt[-0].i;
		  } break;
case 28:
# line 477 "lev_comp.y"
{
		      yyval.i = yypvt[-1].i;
		  } break;
case 96:
# line 560 "lev_comp.y"
{
		      struct lc_vardefs *vd;

		      if ((vd = vardef_defined(vardefs, yypvt[-0].map, 1))) {
			  if (!(vd->var_type & SPOVAR_ARRAY))
			      lc_error("Trying to shuffle non-array variable '%s'",
                                       yypvt[-0].map);
		      } else
                          lc_error("Trying to shuffle undefined variable '%s'",
                                   yypvt[-0].map);
		      add_opvars(splev, "so", VA_PASS2(yypvt[-0].map, SPO_SHUFFLE_ARRAY));
		      Free(yypvt[-0].map);
		  } break;
case 97:
# line 576 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-2].map, SPOVAR_INT);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-2].map, SPO_VAR_INIT));
		      Free(yypvt[-2].map);
		  } break;
case 98:
# line 582 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-4].map, SPOVAR_SEL);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 99:
# line 588 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-2].map, SPOVAR_STRING);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-2].map, SPO_VAR_INIT));
		      Free(yypvt[-2].map);
		  } break;
case 100:
# line 594 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-4].map, SPOVAR_MAPCHAR);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 101:
# line 600 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-4].map, SPOVAR_MONST);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 102:
# line 606 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-4].map, SPOVAR_OBJ);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 103:
# line 612 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-2].map, SPOVAR_COORD);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-2].map, SPO_VAR_INIT));
		      Free(yypvt[-2].map);
		  } break;
case 104:
# line 618 "lev_comp.y"
{
		      vardefs = add_vardef_type(vardefs, yypvt[-2].map, SPOVAR_REGION);
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-2].map, SPO_VAR_INIT));
		      Free(yypvt[-2].map);
		  } break;
case 105:
# line 624 "lev_comp.y"
{
		      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-4].map,
                                                SPOVAR_INT | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 106:
# line 634 "lev_comp.y"
{
		      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-4].map,
                                                SPOVAR_COORD | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 107:
# line 644 "lev_comp.y"
{
                      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-4].map,
                                                SPOVAR_REGION | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 108:
# line 654 "lev_comp.y"
{
                      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-6].map,
                                                SPOVAR_MAPCHAR | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-6].map, SPO_VAR_INIT));
		      Free(yypvt[-6].map);
		  } break;
case 109:
# line 664 "lev_comp.y"
{
		      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-6].map,
                                                SPOVAR_MONST | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-6].map, SPO_VAR_INIT));
		      Free(yypvt[-6].map);
		  } break;
case 110:
# line 674 "lev_comp.y"
{
                      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-6].map,
                                                SPOVAR_OBJ | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-6].map, SPO_VAR_INIT));
		      Free(yypvt[-6].map);
		  } break;
case 111:
# line 684 "lev_comp.y"
{
                      int n_items = (int) yypvt[-1].i;

		      vardefs = add_vardef_type(vardefs, yypvt[-4].map,
                                                SPOVAR_STRING | SPOVAR_ARRAY);
		      add_opvars(splev, "iso",
				 VA_PASS3(n_items, yypvt[-4].map, SPO_VAR_INIT));
		      Free(yypvt[-4].map);
		  } break;
case 112:
# line 696 "lev_comp.y"
{
		      add_opvars(splev, "O", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1;
		  } break;
case 113:
# line 701 "lev_comp.y"
{
		      add_opvars(splev, "O", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 114:
# line 708 "lev_comp.y"
{
		      add_opvars(splev, "M", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1;
		  } break;
case 115:
# line 713 "lev_comp.y"
{
		      add_opvars(splev, "M", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 116:
# line 720 "lev_comp.y"
{
		      add_opvars(splev, "m", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1;
		  } break;
case 117:
# line 725 "lev_comp.y"
{
		      add_opvars(splev, "m", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 118:
# line 732 "lev_comp.y"
{
		      yyval.i = 1;
		  } break;
case 119:
# line 736 "lev_comp.y"
{
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 120:
# line 742 "lev_comp.y"
{
		      add_opvars(splev, "c", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1;
		  } break;
case 121:
# line 747 "lev_comp.y"
{
		      add_opvars(splev, "c", VA_PASS1(yypvt[-0].i));
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 122:
# line 754 "lev_comp.y"
{
		      yyval.i = 1;
		  } break;
case 123:
# line 758 "lev_comp.y"
{
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 124:
# line 764 "lev_comp.y"
{
		      yyval.i = 1;
		  } break;
case 125:
# line 768 "lev_comp.y"
{
		      yyval.i = 1 + yypvt[-2].i;
		  } break;
case 126:
# line 774 "lev_comp.y"
{
		      struct lc_funcdefs *funcdef;

		      if (in_function_definition)
			  lc_error("Recursively defined functions not allowed (function %s).", yypvt[-1].map);

		      in_function_definition++;

		      if (funcdef_defined(function_definitions, yypvt[-1].map, 1))
			  lc_error("Function '%s' already defined once.", yypvt[-1].map);

		      funcdef = funcdef_new(-1, yypvt[-1].map);
		      funcdef->next = function_definitions;
		      function_definitions = funcdef;
		      function_splev_backup = splev;
		      splev = &(funcdef->code);
		      Free(yypvt[-1].map);
		      curr_function = funcdef;
		      function_tmp_var_defs = vardefs;
		      vardefs = NULL;
		  } break;
case 127:
# line 796 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 128:
# line 800 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(0, SPO_RETURN));
		      splev = function_splev_backup;
		      in_function_definition--;
		      curr_function = NULL;
		      vardef_free_all(vardefs);
		      vardefs = function_tmp_var_defs;
		  } break;
case 129:
# line 811 "lev_comp.y"
{
		      struct lc_funcdefs *tmpfunc;

		      tmpfunc = funcdef_defined(function_definitions, yypvt[-3].map, 1);
		      if (tmpfunc) {
			  int l;
			  int nparams = (int) strlen(yypvt[-1].map);
			  char *fparamstr = funcdef_paramtypes(tmpfunc);

			  if (strcmp(yypvt[-1].map, fparamstr)) {
			      char *tmps = strdup(decode_parm_str(fparamstr));

			      lc_error("Function '%s' requires params '%s', got '%s' instead.",
                                       yypvt[-3].map, tmps, decode_parm_str(yypvt[-1].map));
			      Free(tmps);
			  }
			  Free(fparamstr);
			  Free(yypvt[-1].map);
			  if (!(tmpfunc->n_called)) {
			      /* we haven't called the function yet, so insert it in the code */
			      struct opvar *jmp = New(struct opvar);

			      set_opvar_int(jmp, splev->n_opcodes+1);
			      add_opcode(splev, SPO_PUSH, jmp);
                              /* we must jump past it first, then CALL it, due to RETURN. */
			      add_opcode(splev, SPO_JMP, NULL);

			      tmpfunc->addr = splev->n_opcodes;

			      { /* init function parameter variables */
				  struct lc_funcdefs_parm *tfp = tmpfunc->params;
				  while (tfp) {
				      add_opvars(splev, "iso",
						 VA_PASS3(0, tfp->name,
							  SPO_VAR_INIT));
				      tfp = tfp->next;
				  }
			      }

			      splev_add_from(splev, &(tmpfunc->code));
			      set_opvar_int(jmp,
                                            splev->n_opcodes - jmp->vardata.l);
			  }
			  l = (int) (tmpfunc->addr - splev->n_opcodes - 2);
			  add_opvars(splev, "iio",
				     VA_PASS3(nparams, l, SPO_CALL));
			  tmpfunc->n_called++;
		      } else {
			  lc_error("Function '%s' not defined.", yypvt[-3].map);
		      }
		      Free(yypvt[-3].map);
		  } break;
case 130:
# line 866 "lev_comp.y"
{
		      add_opcode(splev, SPO_EXIT, NULL);
		  } break;
case 131:
# line 872 "lev_comp.y"
{
		      yyval.i = 100;
		  } break;
case 132:
# line 876 "lev_comp.y"
{
		      yyval.i = yypvt[-0].i;
		  } break;
case 133:
# line 882 "lev_comp.y"
{
		      /* val > rn2(100) */
		      add_opvars(splev, "iio",
				 VA_PASS3((int) yypvt[-0].i, 100, SPO_RN2));
		      yyval.i = SPO_JG;
                  } break;
case 134:
# line 889 "lev_comp.y"
{
		      yyval.i = yypvt[-2].i;
                  } break;
case 135:
# line 893 "lev_comp.y"
{
		      /* boolean, explicit foo != 0 */
		      add_opvars(splev, "i", VA_PASS1(0));
		      yyval.i = SPO_JNE;
                  } break;
case 136:
# line 901 "lev_comp.y"
{
		      is_inconstant_number = 0;
		  } break;
case 137:
# line 905 "lev_comp.y"
{
		      struct opvar *chkjmp;

		      if (in_switch_statement > 0)
			  lc_error("Cannot nest switch-statements.");

		      in_switch_statement++;

		      n_switch_case_list = 0;
		      switch_default_case = NULL;

		      if (!is_inconstant_number)
			  add_opvars(splev, "o", VA_PASS1(SPO_RN2));
		      is_inconstant_number = 0;

		      chkjmp = New(struct opvar);
		      set_opvar_int(chkjmp, splev->n_opcodes+1);
		      switch_check_jump = chkjmp;
		      add_opcode(splev, SPO_PUSH, chkjmp);
		      add_opcode(splev, SPO_JMP, NULL);
		      break_stmt_start();
		  } break;
case 138:
# line 928 "lev_comp.y"
{
		      struct opvar *endjump = New(struct opvar);
		      int i;

		      set_opvar_int(endjump, splev->n_opcodes+1);

		      add_opcode(splev, SPO_PUSH, endjump);
		      add_opcode(splev, SPO_JMP, NULL);

		      set_opvar_int(switch_check_jump,
			     splev->n_opcodes - switch_check_jump->vardata.l);

		      for (i = 0; i < n_switch_case_list; i++) {
			  add_opvars(splev, "oio",
				     VA_PASS3(SPO_COPY,
					      switch_case_value[i], SPO_CMP));
			  set_opvar_int(switch_case_list[i],
			 switch_case_list[i]->vardata.l - splev->n_opcodes-1);
			  add_opcode(splev, SPO_PUSH, switch_case_list[i]);
			  add_opcode(splev, SPO_JE, NULL);
		      }

		      if (switch_default_case) {
			  set_opvar_int(switch_default_case,
			 switch_default_case->vardata.l - splev->n_opcodes-1);
			  add_opcode(splev, SPO_PUSH, switch_default_case);
			  add_opcode(splev, SPO_JMP, NULL);
		      }

		      set_opvar_int(endjump, splev->n_opcodes - endjump->vardata.l);

		      break_stmt_end(splev);

		      add_opcode(splev, SPO_POP, NULL); /* get rid of the value in stack */
		      in_switch_statement--;


		  } break;
case 141:
# line 973 "lev_comp.y"
{
		      if (n_switch_case_list < MAX_SWITCH_CASES) {
			  struct opvar *tmppush = New(struct opvar);

			  set_opvar_int(tmppush, splev->n_opcodes);
			  switch_case_value[n_switch_case_list] = yypvt[-1].i;
			  switch_case_list[n_switch_case_list++] = tmppush;
		      } else lc_error("Too many cases in a switch.");
		  } break;
case 142:
# line 983 "lev_comp.y"
{
		  } break;
case 143:
# line 986 "lev_comp.y"
{
		      struct opvar *tmppush = New(struct opvar);

		      if (switch_default_case)
			  lc_error("Switch default case already used.");

		      set_opvar_int(tmppush, splev->n_opcodes);
		      switch_default_case = tmppush;
		  } break;
case 144:
# line 996 "lev_comp.y"
{
		  } break;
case 145:
# line 1001 "lev_comp.y"
{
		      if (!allow_break_statements)
			  lc_error("Cannot use BREAK outside a statement block.");
		      else {
			  break_stmt_new(splev, splev->n_opcodes);
		      }
		  } break;
case 148:
# line 1015 "lev_comp.y"
{
		      char buf[256], buf2[256];

		      if (n_forloops >= MAX_NESTED_IFS) {
			  lc_error("FOR: Too deeply nested loops.");
			  n_forloops = MAX_NESTED_IFS - 1;
		      }

		      /* first, define a variable for the for-loop end value */
		      Sprintf(buf, "%s end", yypvt[-4].map);
		      /* the value of which is already in stack (the 2nd math_expr) */
		      add_opvars(splev, "iso", VA_PASS3(0, buf, SPO_VAR_INIT));

		      vardefs = add_vardef_type(vardefs, yypvt[-4].map, SPOVAR_INT);
		      /* define the for-loop variable. value is in stack (1st math_expr) */
		      add_opvars(splev, "iso", VA_PASS3(0, yypvt[-4].map, SPO_VAR_INIT));

		      /* calculate value for the loop "step" variable */
		      Sprintf(buf2, "%s step", yypvt[-4].map);
		      /* end - start */
		      add_opvars(splev, "vvo",
				 VA_PASS3(buf, yypvt[-4].map, SPO_MATH_SUB));
		      /* sign of that */
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_SIGN));
		      /* save the sign into the step var */
		      add_opvars(splev, "iso",
				 VA_PASS3(0, buf2, SPO_VAR_INIT));

		      forloop_list[n_forloops].varname = strdup(yypvt[-4].map);
		      forloop_list[n_forloops].jmp_point = splev->n_opcodes;

		      n_forloops++;
		      Free(yypvt[-4].map);
		  } break;
case 149:
# line 1052 "lev_comp.y"
{
		      /* nothing */
		      break_stmt_start();
		  } break;
case 150:
# line 1057 "lev_comp.y"
{
                      int l;
		      char buf[256], buf2[256];

		      n_forloops--;
		      Sprintf(buf, "%s step", forloop_list[n_forloops].varname);
		      Sprintf(buf2, "%s end", forloop_list[n_forloops].varname);
		      /* compare for-loop var to end value */
		      add_opvars(splev, "vvo",
				 VA_PASS3(forloop_list[n_forloops].varname,
					  buf2, SPO_CMP));
		      /* var + step */
		      add_opvars(splev, "vvo",
				VA_PASS3(buf, forloop_list[n_forloops].varname,
					 SPO_MATH_ADD));
		      /* for-loop var = (for-loop var + step) */
		      add_opvars(splev, "iso",
				 VA_PASS3(0, forloop_list[n_forloops].varname,
					  SPO_VAR_INIT));
		      /* jump back if compared values were not equal */
                      l = (int) (forloop_list[n_forloops].jmp_point
                                 - splev->n_opcodes - 1);
		      add_opvars(splev, "io", VA_PASS2(l, SPO_JNE));
		      Free(forloop_list[n_forloops].varname);
		      break_stmt_end(splev);
		  } break;
case 151:
# line 1086 "lev_comp.y"
{
		      struct opvar *tmppush = New(struct opvar);

		      if (n_if_list >= MAX_NESTED_IFS) {
			  lc_error("LOOP: Too deeply nested conditionals.");
			  n_if_list = MAX_NESTED_IFS - 1;
		      }
		      set_opvar_int(tmppush, splev->n_opcodes);
		      if_list[n_if_list++] = tmppush;

		      add_opvars(splev, "o", VA_PASS1(SPO_DEC));
		      break_stmt_start();
		  } break;
case 152:
# line 1100 "lev_comp.y"
{
		      struct opvar *tmppush;

		      add_opvars(splev, "oio", VA_PASS3(SPO_COPY, 0, SPO_CMP));

		      tmppush = (struct opvar *) if_list[--n_if_list];
		      set_opvar_int(tmppush,
                                    tmppush->vardata.l - splev->n_opcodes-1);
		      add_opcode(splev, SPO_PUSH, tmppush);
		      add_opcode(splev, SPO_JG, NULL);
		      add_opcode(splev, SPO_POP, NULL); /* discard count */
		      break_stmt_end(splev);
		  } break;
case 153:
# line 1116 "lev_comp.y"
{
		      struct opvar *tmppush2 = New(struct opvar);

		      if (n_if_list >= MAX_NESTED_IFS) {
			  lc_error("IF: Too deeply nested conditionals.");
			  n_if_list = MAX_NESTED_IFS - 1;
		      }

		      add_opcode(splev, SPO_CMP, NULL);

		      set_opvar_int(tmppush2, splev->n_opcodes+1);

		      if_list[n_if_list++] = tmppush2;

		      add_opcode(splev, SPO_PUSH, tmppush2);

		      add_opcode(splev, reverse_jmp_opcode( yypvt[-1].i ), NULL);

		  } break;
case 154:
# line 1136 "lev_comp.y"
{
		      if (n_if_list > 0) {
			  struct opvar *tmppush;

			  tmppush = (struct opvar *) if_list[--n_if_list];
			  set_opvar_int(tmppush,
                                        splev->n_opcodes - tmppush->vardata.l);
		      } else lc_error("IF: Huh?!  No start address?");
		  } break;
case 155:
# line 1148 "lev_comp.y"
{
		      struct opvar *tmppush2 = New(struct opvar);

		      if (n_if_list >= MAX_NESTED_IFS) {
			  lc_error("IF: Too deeply nested conditionals.");
			  n_if_list = MAX_NESTED_IFS - 1;
		      }

		      add_opcode(splev, SPO_CMP, NULL);

		      set_opvar_int(tmppush2, splev->n_opcodes+1);

		      if_list[n_if_list++] = tmppush2;

		      add_opcode(splev, SPO_PUSH, tmppush2);

		      add_opcode(splev, reverse_jmp_opcode( yypvt[-0].i ), NULL);

		  } break;
case 156:
# line 1168 "lev_comp.y"
{
		     /* do nothing */
		  } break;
case 157:
# line 1174 "lev_comp.y"
{
		      if (n_if_list > 0) {
			  struct opvar *tmppush;

			  tmppush = (struct opvar *) if_list[--n_if_list];
			  set_opvar_int(tmppush,
                                        splev->n_opcodes - tmppush->vardata.l);
		      } else lc_error("IF: Huh?!  No start address?");
		  } break;
case 158:
# line 1184 "lev_comp.y"
{
		      if (n_if_list > 0) {
			  struct opvar *tmppush = New(struct opvar);
			  struct opvar *tmppush2;

			  set_opvar_int(tmppush, splev->n_opcodes+1);
			  add_opcode(splev, SPO_PUSH, tmppush);

			  add_opcode(splev, SPO_JMP, NULL);

			  tmppush2 = (struct opvar *) if_list[--n_if_list];

			  set_opvar_int(tmppush2,
                                      splev->n_opcodes - tmppush2->vardata.l);
			  if_list[n_if_list++] = tmppush;
		      } else lc_error("IF: Huh?!  No else-part address?");
		  } break;
case 159:
# line 1202 "lev_comp.y"
{
		      if (n_if_list > 0) {
			  struct opvar *tmppush;
			  tmppush = (struct opvar *) if_list[--n_if_list];
			  set_opvar_int(tmppush, splev->n_opcodes - tmppush->vardata.l);
		      } else lc_error("IF: Huh?! No end address?");
		  } break;
case 160:
# line 1212 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MESSAGE));
		  } break;
case 161:
# line 1218 "lev_comp.y"
{
		      add_opvars(splev, "iiiiiio",
			      VA_PASS7(-1,  0, -1, -1, -1, -1, SPO_CORRIDOR));
		  } break;
case 162:
# line 1223 "lev_comp.y"
{
		      add_opvars(splev, "iiiiiio",
			      VA_PASS7(-1, yypvt[-0].i, -1, -1, -1, -1, SPO_CORRIDOR));
		  } break;
case 163:
# line 1228 "lev_comp.y"
{
		      add_opvars(splev, "iiiiiio",
			      VA_PASS7(-1, -1, -1, -1, -1, -1, SPO_CORRIDOR));
		  } break;
case 164:
# line 1235 "lev_comp.y"
{
		      add_opvars(splev, "iiiiiio",
				 VA_PASS7(yypvt[-2].corpos.room, yypvt[-2].corpos.door, yypvt[-2].corpos.wall,
					  yypvt[-0].corpos.room, yypvt[-0].corpos.door, yypvt[-0].corpos.wall,
					  SPO_CORRIDOR));
		  } break;
case 165:
# line 1242 "lev_comp.y"
{
		      add_opvars(splev, "iiiiiio",
				 VA_PASS7(yypvt[-2].corpos.room, yypvt[-2].corpos.door, yypvt[-2].corpos.wall,
					  -1, -1, (long)yypvt[-0].i,
					  SPO_CORRIDOR));
		  } break;
case 166:
# line 1251 "lev_comp.y"
{
			yyval.corpos.room = yypvt[-5].i;
			yyval.corpos.wall = yypvt[-3].i;
			yyval.corpos.door = yypvt[-1].i;
		  } break;
case 167:
# line 1259 "lev_comp.y"
{
		      if ((yypvt[-2].i < 100) && (yypvt[-3].i == OROOM))
			  lc_error("Only typed rooms can have a chance.");
		      else {
			  add_opvars(splev, "iii",
				     VA_PASS3((long)yypvt[-3].i, (long)yypvt[-2].i, (long)yypvt[-0].i));
		      }
                  } break;
case 168:
# line 1270 "lev_comp.y"
{
		      long rflags = yypvt[-0].i;

		      if (rflags == -1) rflags = (1 << 0);
		      add_opvars(splev, "iiiiiiio",
				 VA_PASS8(rflags, ERR, ERR,
					  yypvt[-3].crd.x, yypvt[-3].crd.y, yypvt[-1].sze.width, yypvt[-1].sze.height,
					  SPO_SUBROOM));
		      break_stmt_start();
		  } break;
case 169:
# line 1281 "lev_comp.y"
{
		      break_stmt_end(splev);
		      add_opcode(splev, SPO_ENDROOM, NULL);
		  } break;
case 170:
# line 1288 "lev_comp.y"
{
		      long rflags = yypvt[-2].i;

		      if (rflags == -1) rflags = (1 << 0);
		      add_opvars(splev, "iiiiiiio",
				 VA_PASS8(rflags,
					  yypvt[-3].crd.x, yypvt[-3].crd.y, yypvt[-5].crd.x, yypvt[-5].crd.y,
					  yypvt[-1].sze.width, yypvt[-1].sze.height, SPO_ROOM));
		      break_stmt_start();
		  } break;
case 171:
# line 1299 "lev_comp.y"
{
		      break_stmt_end(splev);
		      add_opcode(splev, SPO_ENDROOM, NULL);
		  } break;
case 172:
# line 1306 "lev_comp.y"
{
			yyval.i = 1;
		  } break;
case 173:
# line 1310 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 174:
# line 1316 "lev_comp.y"
{
			if ( yypvt[-3].i < 1 || yypvt[-3].i > 5 ||
			    yypvt[-1].i < 1 || yypvt[-1].i > 5 ) {
			    lc_error("Room positions should be between 1-5: (%li,%li)!", yypvt[-3].i, yypvt[-1].i);
			} else {
			    yyval.crd.x = yypvt[-3].i;
			    yyval.crd.y = yypvt[-1].i;
			}
		  } break;
case 175:
# line 1326 "lev_comp.y"
{
			yyval.crd.x = yyval.crd.y = ERR;
		  } break;
case 176:
# line 1332 "lev_comp.y"
{
			if ( yypvt[-3].i < 0 || yypvt[-1].i < 0) {
			    lc_error("Invalid subroom position (%li,%li)!", yypvt[-3].i, yypvt[-1].i);
			} else {
			    yyval.crd.x = yypvt[-3].i;
			    yyval.crd.y = yypvt[-1].i;
			}
		  } break;
case 177:
# line 1341 "lev_comp.y"
{
			yyval.crd.x = yyval.crd.y = ERR;
		  } break;
case 178:
# line 1347 "lev_comp.y"
{
		      yyval.crd.x = yypvt[-3].i;
		      yyval.crd.y = yypvt[-1].i;
		  } break;
case 179:
# line 1352 "lev_comp.y"
{
		      yyval.crd.x = yyval.crd.y = ERR;
		  } break;
case 180:
# line 1358 "lev_comp.y"
{
			yyval.sze.width = yypvt[-3].i;
			yyval.sze.height = yypvt[-1].i;
		  } break;
case 181:
# line 1363 "lev_comp.y"
{
			yyval.sze.height = yyval.sze.width = ERR;
		  } break;
case 182:
# line 1369 "lev_comp.y"
{
			/* ERR means random here */
			if (yypvt[-2].i == ERR && yypvt[-0].i != ERR) {
			    lc_error("If the door wall is random, so must be its pos!");
			} else {
			    add_opvars(splev, "iiiio",
				       VA_PASS5((long)yypvt[-0].i, (long)yypvt[-4].i, (long)yypvt[-6].i,
						(long)yypvt[-2].i, SPO_ROOM_DOOR));
			}
		  } break;
case 183:
# line 1380 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2((long)yypvt[-2].i, SPO_DOOR));
		  } break;
case 188:
# line 1394 "lev_comp.y"
{
		      yyval.i = yypvt[-0].i;
		  } break;
case 189:
# line 1398 "lev_comp.y"
{
		      yyval.i = (yypvt[-2].i | yypvt[-0].i);
		  } break;
case 192:
# line 1408 "lev_comp.y"
{
		      add_opvars(splev, "ciisiio",
				 VA_PASS7(0, 0, 1, (char *) 0, 0, 0, SPO_MAP));
		      max_x_map = COLNO-1;
		      max_y_map = ROWNO;
		  } break;
case 193:
# line 1415 "lev_comp.y"
{
		      add_opvars(splev, "cii",
				 VA_PASS3(SP_COORD_PACK((yypvt[-4].i), (yypvt[-2].i)),
					  1, (int) yypvt[-1].i));
		      scan_map(yypvt[-0].map, splev);
		      Free(yypvt[-0].map);
		  } break;
case 194:
# line 1423 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(2, (int) yypvt[-1].i));
		      scan_map(yypvt[-0].map, splev);
		      Free(yypvt[-0].map);
		  } break;
case 199:
# line 1439 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(0, SPO_MONSTER));
		  } break;
case 200:
# line 1443 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(1, SPO_MONSTER));
		      in_container_obj++;
		      break_stmt_start();
		  } break;
case 201:
# line 1449 "lev_comp.y"
{
		     break_stmt_end(splev);
		     in_container_obj--;
		     add_opvars(splev, "o", VA_PASS1(SPO_END_MONINVENT));
		 } break;
case 202:
# line 1457 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 203:
# line 1463 "lev_comp.y"
{
		      struct opvar *stopit = New(struct opvar);

		      set_opvar_int(stopit, SP_M_V_END);
		      add_opcode(splev, SPO_PUSH, stopit);
		      yyval.i = 0x0000;
		  } break;
case 204:
# line 1471 "lev_comp.y"
{
		      if (( yypvt[-2].i & yypvt[-0].i ))
			  lc_error("MONSTER extra info defined twice.");
		      yyval.i = ( yypvt[-2].i | yypvt[-0].i );
		  } break;
case 205:
# line 1479 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_M_V_NAME));
		      yyval.i = 0x0001;
		  } break;
case 206:
# line 1484 "lev_comp.y"
{
		      add_opvars(splev, "ii",
				 VA_PASS2((int) yypvt[-0].i, SP_M_V_PEACEFUL));
		      yyval.i = 0x0002;
		  } break;
case 207:
# line 1490 "lev_comp.y"
{
		      add_opvars(splev, "ii",
				 VA_PASS2((int) yypvt[-0].i, SP_M_V_ASLEEP));
		      yyval.i = 0x0004;
		  } break;
case 208:
# line 1496 "lev_comp.y"
{
		      add_opvars(splev, "ii",
				 VA_PASS2((int) yypvt[-0].i, SP_M_V_ALIGN));
		      yyval.i = 0x0008;
		  } break;
case 209:
# line 1502 "lev_comp.y"
{
		      add_opvars(splev, "ii",
				 VA_PASS2((int) yypvt[-1].i, SP_M_V_APPEAR));
		      yyval.i = 0x0010;
		  } break;
case 210:
# line 1508 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_FEMALE));
		      yyval.i = 0x0020;
		  } break;
case 211:
# line 1513 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_INVIS));
		      yyval.i = 0x0040;
		  } break;
case 212:
# line 1518 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_CANCELLED));
		      yyval.i = 0x0080;
		  } break;
case 213:
# line 1523 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_REVIVED));
		      yyval.i = 0x0100;
		  } break;
case 214:
# line 1528 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_AVENGE));
		      yyval.i = 0x0200;
		  } break;
case 215:
# line 1533 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_M_V_FLEEING));
		      yyval.i = 0x0400;
		  } break;
case 216:
# line 1538 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_M_V_BLINDED));
		      yyval.i = 0x0800;
		  } break;
case 217:
# line 1543 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_M_V_PARALYZED));
		      yyval.i = 0x1000;
		  } break;
case 218:
# line 1548 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_STUNNED));
		      yyval.i = 0x2000;
		  } break;
case 219:
# line 1553 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_M_V_CONFUSED));
		      yyval.i = 0x4000;
		  } break;
case 220:
# line 1558 "lev_comp.y"
{
		      add_opvars(splev, "ii",
				 VA_PASS2((int) yypvt[-0].i, SP_M_V_SEENTRAPS));
		      yyval.i = 0x8000;
		  } break;
case 221:
# line 1566 "lev_comp.y"
{
		      int token = get_trap_type(yypvt[-0].map);

		      if (token == ERR || token == 0)
			  lc_error("Unknown trap type '%s'!", yypvt[-0].map);
                      Free(yypvt[-0].map);
		      yyval.i = (1L << (token - 1));
		  } break;
case 222:
# line 1575 "lev_comp.y"
{
		      yyval.i = (long) ~0;
		  } break;
case 223:
# line 1579 "lev_comp.y"
{
		      int token = get_trap_type(yypvt[-2].map);
		      if (token == ERR || token == 0)
			  lc_error("Unknown trap type '%s'!", yypvt[-2].map);

		      if ((1L << (token - 1)) & yypvt[-0].i)
			  lc_error("Monster seen_traps, trap '%s' listed twice.", yypvt[-2].map);
                      Free(yypvt[-2].map);
		      yyval.i = ((1L << (token - 1)) | yypvt[-0].i);
		  } break;
case 224:
# line 1592 "lev_comp.y"
{
		      int cnt = 0;

		      if (in_container_obj)
                          cnt |= SP_OBJ_CONTENT;
		      add_opvars(splev, "io", VA_PASS2(cnt, SPO_OBJECT));
		  } break;
case 225:
# line 1600 "lev_comp.y"
{
		      int cnt = SP_OBJ_CONTAINER;

		      if (in_container_obj)
                          cnt |= SP_OBJ_CONTENT;
		      add_opvars(splev, "io", VA_PASS2(cnt, SPO_OBJECT));
		      in_container_obj++;
		      break_stmt_start();
		  } break;
case 226:
# line 1610 "lev_comp.y"
{
		     break_stmt_end(splev);
		     in_container_obj--;
		     add_opcode(splev, SPO_POP_CONTAINER, NULL);
		 } break;
case 227:
# line 1618 "lev_comp.y"
{
		      if (( yypvt[-0].i & 0x4000) && in_container_obj)
                          lc_error("Object cannot have a coord when contained.");
		      else if (!( yypvt[-0].i & 0x4000) && !in_container_obj)
                          lc_error("Object needs a coord when not contained.");
		  } break;
case 228:
# line 1627 "lev_comp.y"
{
		      struct opvar *stopit = New(struct opvar);
		      set_opvar_int(stopit, SP_O_V_END);
		      add_opcode(splev, SPO_PUSH, stopit);
		      yyval.i = 0x00;
		  } break;
case 229:
# line 1634 "lev_comp.y"
{
		      if (( yypvt[-2].i & yypvt[-0].i ))
			  lc_error("OBJECT extra info '%s' defined twice.", curr_token);
		      yyval.i = ( yypvt[-2].i | yypvt[-0].i );
		  } break;
case 230:
# line 1642 "lev_comp.y"
{
		      add_opvars(splev, "ii",
				 VA_PASS2((int) yypvt[-0].i, SP_O_V_CURSE));
		      yyval.i = 0x0001;
		  } break;
case 231:
# line 1648 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_CORPSENM));
		      yyval.i = 0x0002;
		  } break;
case 232:
# line 1653 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_SPE));
		      yyval.i = 0x0004;
		  } break;
case 233:
# line 1658 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_NAME));
		      yyval.i = 0x0008;
		  } break;
case 234:
# line 1663 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_QUAN));
		      yyval.i = 0x0010;
		  } break;
case 235:
# line 1668 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_O_V_BURIED));
		      yyval.i = 0x0020;
		  } break;
case 236:
# line 1673 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2((int) yypvt[-0].i, SP_O_V_LIT));
		      yyval.i = 0x0040;
		  } break;
case 237:
# line 1678 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_ERODED));
		      yyval.i = 0x0080;
		  } break;
case 238:
# line 1683 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(-1, SP_O_V_ERODED));
		      yyval.i = 0x0080;
		  } break;
case 239:
# line 1688 "lev_comp.y"
{
		      if (yypvt[-0].i == D_LOCKED) {
			  add_opvars(splev, "ii", VA_PASS2(1, SP_O_V_LOCKED));
			  yyval.i = 0x0100;
		      } else if (yypvt[-0].i == D_BROKEN) {
			  add_opvars(splev, "ii", VA_PASS2(1, SP_O_V_BROKEN));
			  yyval.i = 0x0200;
		      } else
			  lc_error("DOOR state can only be locked or broken.");
		  } break;
case 240:
# line 1699 "lev_comp.y"
{
		      add_opvars(splev, "ii",
                                 VA_PASS2((int) yypvt[-0].i, SP_O_V_TRAPPED));
		      yyval.i = 0x0400;
		  } break;
case 241:
# line 1705 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_RECHARGED));
		      yyval.i = 0x0800;
		  } break;
case 242:
# line 1710 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_O_V_INVIS));
		      yyval.i = 0x1000;
		  } break;
case 243:
# line 1715 "lev_comp.y"
{
		      add_opvars(splev, "ii", VA_PASS2(1, SP_O_V_GREASED));
		      yyval.i = 0x2000;
		  } break;
case 244:
# line 1720 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(SP_O_V_COORD));
		      yyval.i = 0x4000;
		  } break;
case 245:
# line 1727 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2((int) yypvt[-2].i, SPO_TRAP));
		  } break;
case 246:
# line 1733 "lev_comp.y"
{
		       long dir, state = 0;

		       /* convert dir from a DIRECTION to a DB_DIR */
		       dir = yypvt[-2].i;
		       switch (dir) {
		       case W_NORTH: dir = DB_NORTH; break;
		       case W_SOUTH: dir = DB_SOUTH; break;
		       case W_EAST:  dir = DB_EAST;  break;
		       case W_WEST:  dir = DB_WEST;  break;
		       default:
			   lc_error("Invalid drawbridge direction.");
			   break;
		       }

		       if ( yypvt[-0].i == D_ISOPEN )
			   state = 1;
		       else if ( yypvt[-0].i == D_CLOSED )
			   state = 0;
		       else if ( yypvt[-0].i == -1 )
			   state = -1;
		       else
			   lc_error("A drawbridge can only be open, closed or random!");
		       add_opvars(splev, "iio",
				  VA_PASS3(state, dir, SPO_DRAWBRIDGE));
		   } break;
case 247:
# line 1762 "lev_comp.y"
{
		      add_opvars(splev, "iiio",
				 VA_PASS4((int) yypvt[-0].i, 1, 0, SPO_MAZEWALK));
		  } break;
case 248:
# line 1767 "lev_comp.y"
{
		      add_opvars(splev, "iiio",
				 VA_PASS4((int) yypvt[-3].i, (int) yypvt[-1].i,
					  (int) yypvt[-0].i, SPO_MAZEWALK));
		  } break;
case 249:
# line 1775 "lev_comp.y"
{
		      add_opvars(splev, "rio",
				 VA_PASS3(SP_REGION_PACK(-1,-1,-1,-1),
					  0, SPO_WALLIFY));
		  } break;
case 250:
# line 1781 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(1, SPO_WALLIFY));
		  } break;
case 251:
# line 1787 "lev_comp.y"
{
		      add_opvars(splev, "io",
				 VA_PASS2((int) yypvt[-0].i, SPO_LADDER));
		  } break;
case 252:
# line 1794 "lev_comp.y"
{
		      add_opvars(splev, "io",
				 VA_PASS2((int) yypvt[-0].i, SPO_STAIR));
		  } break;
case 253:
# line 1801 "lev_comp.y"
{
		      add_opvars(splev, "iiiii iiiii iiso",
				 VA_PASS14(yypvt[-4].lregn.x1, yypvt[-4].lregn.y1, yypvt[-4].lregn.x2, yypvt[-4].lregn.y2, yypvt[-4].lregn.area,
					   yypvt[-2].lregn.x1, yypvt[-2].lregn.y1, yypvt[-2].lregn.x2, yypvt[-2].lregn.y2, yypvt[-2].lregn.area,
				     (long) ((yypvt[-0].i) ? LR_UPSTAIR : LR_DOWNSTAIR),
					   0, (char *) 0, SPO_LEVREGION));
		  } break;
case 254:
# line 1811 "lev_comp.y"
{
		      add_opvars(splev, "iiiii iiiii iiso",
				 VA_PASS14(yypvt[-4].lregn.x1, yypvt[-4].lregn.y1, yypvt[-4].lregn.x2, yypvt[-4].lregn.y2, yypvt[-4].lregn.area,
					   yypvt[-2].lregn.x1, yypvt[-2].lregn.y1, yypvt[-2].lregn.x2, yypvt[-2].lregn.y2, yypvt[-2].lregn.area,
					   LR_PORTAL, 0, yypvt[-0].map, SPO_LEVREGION));
		      Free(yypvt[-0].map);
		  } break;
case 255:
# line 1821 "lev_comp.y"
{
		      long rtyp = 0;
		      switch(yypvt[-0].i) {
		      case -1: rtyp = LR_TELE; break;
		      case  0: rtyp = LR_DOWNTELE; break;
		      case  1: rtyp = LR_UPTELE; break;
		      }
		      add_opvars(splev, "iiiii iiiii iiso",
				 VA_PASS14(yypvt[-3].lregn.x1, yypvt[-3].lregn.y1, yypvt[-3].lregn.x2, yypvt[-3].lregn.y2, yypvt[-3].lregn.area,
					   yypvt[-1].lregn.x1, yypvt[-1].lregn.y1, yypvt[-1].lregn.x2, yypvt[-1].lregn.y2, yypvt[-1].lregn.area,
					   rtyp, 0, (char *)0, SPO_LEVREGION));
		  } break;
case 256:
# line 1836 "lev_comp.y"
{
		      add_opvars(splev, "iiiii iiiii iiso",
				 VA_PASS14(yypvt[-2].lregn.x1, yypvt[-2].lregn.y1, yypvt[-2].lregn.x2, yypvt[-2].lregn.y2, yypvt[-2].lregn.area,
					   yypvt[-0].lregn.x1, yypvt[-0].lregn.y1, yypvt[-0].lregn.x2, yypvt[-0].lregn.y2, yypvt[-0].lregn.area,
					   (long) LR_BRANCH, 0,
					   (char *) 0, SPO_LEVREGION));
		  } break;
case 257:
# line 1846 "lev_comp.y"
{
			yyval.i = -1;
		  } break;
case 258:
# line 1850 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 259:
# line 1856 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_FOUNTAIN));
		  } break;
case 260:
# line 1862 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SINK));
		  } break;
case 261:
# line 1868 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_POOL));
		  } break;
case 262:
# line 1874 "lev_comp.y"
{
		      yyval.terr.lit = -2;
		      yyval.terr.ter = what_map_char((char) yypvt[-0].i);
		  } break;
case 263:
# line 1879 "lev_comp.y"
{
		      yyval.terr.lit = yypvt[-1].i;
		      yyval.terr.ter = what_map_char((char) yypvt[-3].i);
		  } break;
case 264:
# line 1886 "lev_comp.y"
{
		      add_opvars(splev, "io",
				 VA_PASS2(yypvt[-0].i, SPO_REPLACETERRAIN));
		  } break;
case 265:
# line 1893 "lev_comp.y"
{
		     add_opvars(splev, "o", VA_PASS1(SPO_TERRAIN));
		 } break;
case 266:
# line 1899 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_NON_DIGGABLE));
		  } break;
case 267:
# line 1905 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_NON_PASSWALL));
		  } break;
case 268:
# line 1911 "lev_comp.y"
{
		      long irr;
		      long rt = yypvt[-1].i;
		      long rflags = yypvt[-0].i;

		      if (rflags == -1) rflags = (1 << 0);
		      if (!(rflags & 1)) rt += MAXRTYPE+1;
		      irr = ((rflags & 2) != 0);
		      add_opvars(splev, "iiio",
				 VA_PASS4((long)yypvt[-3].i, rt, rflags, SPO_REGION));
		      yyval.i = (irr || (rflags & 1) || rt != OROOM);
		      break_stmt_start();
		  } break;
case 269:
# line 1925 "lev_comp.y"
{
		      break_stmt_end(splev);
		      if ( yypvt[-1].i ) {
			  add_opcode(splev, SPO_ENDROOM, NULL);
		      } else if ( yypvt[-0].i )
			  lc_error("Cannot use lev statements in non-permanent REGION");
		  } break;
case 270:
# line 1935 "lev_comp.y"
{
		      yyval.i = 0;
		  } break;
case 271:
# line 1939 "lev_comp.y"
{
		      yyval.i = yypvt[-0].i;
		  } break;
case 272:
# line 1945 "lev_comp.y"
{
		      add_opvars(splev, "iio",
				 VA_PASS3((long)yypvt[-0].i, (long)yypvt[-2].i, SPO_ALTAR));
		  } break;
case 273:
# line 1952 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(2, SPO_GRAVE));
		  } break;
case 274:
# line 1956 "lev_comp.y"
{
		      add_opvars(splev, "sio",
				 VA_PASS3((char *)0, 1, SPO_GRAVE));
		  } break;
case 275:
# line 1961 "lev_comp.y"
{
		      add_opvars(splev, "sio",
				 VA_PASS3((char *)0, 0, SPO_GRAVE));
		  } break;
case 276:
# line 1968 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_GOLD));
		  } break;
case 277:
# line 1974 "lev_comp.y"
{
		      add_opvars(splev, "io",
				 VA_PASS2((long)yypvt[-2].i, SPO_ENGRAVING));
		  } break;
case 278:
# line 1981 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MINERALIZE));
		  } break;
case 279:
# line 1985 "lev_comp.y"
{
		      add_opvars(splev, "iiiio",
				 VA_PASS5(-1L, -1L, -1L, -1L, SPO_MINERALIZE));
		  } break;
case 280:
# line 1992 "lev_comp.y"
{
			int token = get_trap_type(yypvt[-0].map);
			if (token == ERR)
			    lc_error("Unknown trap type '%s'!", yypvt[-0].map);
			yyval.i = token;
			Free(yypvt[-0].map);
		  } break;
case 282:
# line 2003 "lev_comp.y"
{
			int token = get_room_type(yypvt[-0].map);
			if (token == ERR) {
			    lc_warning("Unknown room type \"%s\"!  Making ordinary room...", yypvt[-0].map);
				yyval.i = OROOM;
			} else
				yyval.i = token;
			Free(yypvt[-0].map);
		  } break;
case 284:
# line 2016 "lev_comp.y"
{
			yyval.i = -1;
		  } break;
case 285:
# line 2020 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 286:
# line 2026 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 287:
# line 2030 "lev_comp.y"
{
			yyval.i = yypvt[-2].i | yypvt[-0].i;
		  } break;
case 288:
# line 2037 "lev_comp.y"
{
		      yyval.i = (yypvt[-0].i << 0);
		  } break;
case 289:
# line 2041 "lev_comp.y"
{
		      yyval.i = (yypvt[-0].i << 1);
		  } break;
case 290:
# line 2045 "lev_comp.y"
{
		      yyval.i = (yypvt[-0].i << 2);
		  } break;
case 297:
# line 2061 "lev_comp.y"
{
			yyval.i = - MAX_REGISTERS - 1;
		  } break;
case 300:
# line 2069 "lev_comp.y"
{
			yyval.i = - MAX_REGISTERS - 1;
		  } break;
case 303:
# line 2079 "lev_comp.y"
{
			if ( yypvt[-1].i >= 3 )
				lc_error("Register Index overflow!");
			else
				yyval.i = - yypvt[-1].i - 1;
		  } break;
case 304:
# line 2088 "lev_comp.y"
{
		      add_opvars(splev, "s", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 305:
# line 2093 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_STRING);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 306:
# line 2100 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
                                        SPOVAR_STRING | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		  } break;
case 307:
# line 2111 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 308:
# line 2117 "lev_comp.y"
{
		      add_opvars(splev, "c", VA_PASS1(yypvt[-0].i));
		  } break;
case 309:
# line 2121 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_RNDCOORD));
		  } break;
case 310:
# line 2125 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_COORD);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 311:
# line 2132 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
                                        SPOVAR_COORD | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		  } break;
case 312:
# line 2142 "lev_comp.y"
{
		      if (yypvt[-3].i < 0 || yypvt[-1].i < 0 || yypvt[-3].i >= COLNO || yypvt[-1].i >= ROWNO)
                          lc_error("Coordinates (%li,%li) out of map range!",
                                   yypvt[-3].i, yypvt[-1].i);
		      yyval.i = SP_COORD_PACK(yypvt[-3].i, yypvt[-1].i);
		  } break;
case 313:
# line 2149 "lev_comp.y"
{
		      yyval.i = SP_COORD_PACK_RANDOM(0);
		  } break;
case 314:
# line 2153 "lev_comp.y"
{
		      yyval.i = SP_COORD_PACK_RANDOM(yypvt[-1].i);
		  } break;
case 315:
# line 2159 "lev_comp.y"
{
		      yyval.i = yypvt[-0].i;
		  } break;
case 316:
# line 2163 "lev_comp.y"
{
		      if ((yypvt[-2].i & yypvt[-0].i))
			  lc_warning("Humidity flag used twice.");
		      yyval.i = (yypvt[-2].i | yypvt[-0].i);
		  } break;
case 317:
# line 2171 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 318:
# line 2175 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_REGION);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 319:
# line 2182 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
                                        SPOVAR_REGION | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		  } break;
case 320:
# line 2192 "lev_comp.y"
{
		      long r = SP_REGION_PACK(yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);

		      if (yypvt[-7].i > yypvt[-3].i || yypvt[-5].i > yypvt[-1].i)
			  lc_error("Region start > end: (%ld,%ld,%ld,%ld)!",
                                   yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);

		      add_opvars(splev, "r", VA_PASS1(r));
		      yyval.i = r;
		  } break;
case 321:
# line 2205 "lev_comp.y"
{
		      add_opvars(splev, "m", VA_PASS1(yypvt[-0].i));
		  } break;
case 322:
# line 2209 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_MAPCHAR);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 323:
# line 2216 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
                                        SPOVAR_MAPCHAR | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		  } break;
case 324:
# line 2226 "lev_comp.y"
{
		      if (what_map_char((char) yypvt[-0].i) != INVALID_TYPE)
			  yyval.i = SP_MAPCHAR_PACK(what_map_char((char) yypvt[-0].i), -2);
		      else {
			  lc_error("Unknown map char type '%c'!", yypvt[-0].i);
			  yyval.i = SP_MAPCHAR_PACK(STONE, -2);
		      }
		  } break;
case 325:
# line 2235 "lev_comp.y"
{
		      if (what_map_char((char) yypvt[-3].i) != INVALID_TYPE)
			  yyval.i = SP_MAPCHAR_PACK(what_map_char((char) yypvt[-3].i), yypvt[-1].i);
		      else {
			  lc_error("Unknown map char type '%c'!", yypvt[-3].i);
			  yyval.i = SP_MAPCHAR_PACK(STONE, yypvt[-1].i);
		      }
		  } break;
case 326:
# line 2246 "lev_comp.y"
{
		      add_opvars(splev, "M", VA_PASS1(yypvt[-0].i));
		  } break;
case 327:
# line 2250 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_MONST);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 328:
# line 2257 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
                                        SPOVAR_MONST | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		  } break;
case 329:
# line 2267 "lev_comp.y"
{
                      long m = get_monster_id(yypvt[-0].map, (char)0);
                      if (m == ERR) {
                          lc_error("Unknown monster \"%s\"!", yypvt[-0].map);
                          yyval.i = -1;
                      } else
                          yyval.i = SP_MONST_PACK(m,
                                         def_monsyms[(int) mons[m].mlet].sym);
                      Free(yypvt[-0].map);
                  } break;
case 330:
# line 2278 "lev_comp.y"
{
                        if (check_monster_char((char) yypvt[-0].i))
                            yyval.i = SP_MONST_PACK(-1, yypvt[-0].i);
                        else {
                            lc_error("Unknown monster class '%c'!", yypvt[-0].i);
                            yyval.i = -1;
                        }
                  } break;
case 331:
# line 2287 "lev_comp.y"
{
                      long m = get_monster_id(yypvt[-1].map, (char) yypvt[-3].i);
                      if (m == ERR) {
                          lc_error("Unknown monster ('%c', \"%s\")!", yypvt[-3].i, yypvt[-1].map);
                          yyval.i = -1;
                      } else
                          yyval.i = SP_MONST_PACK(m, yypvt[-3].i);
                      Free(yypvt[-1].map);
                  } break;
case 332:
# line 2297 "lev_comp.y"
{
                      yyval.i = -1;
                  } break;
case 333:
# line 2303 "lev_comp.y"
{
		      add_opvars(splev, "O", VA_PASS1(yypvt[-0].i));
		  } break;
case 334:
# line 2307 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_OBJ);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 335:
# line 2314 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
                                        SPOVAR_OBJ | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		  } break;
case 336:
# line 2324 "lev_comp.y"
{
		      long m = get_object_id(yypvt[-0].map, (char)0);
		      if (m == ERR) {
			  lc_error("Unknown object \"%s\"!", yypvt[-0].map);
			  yyval.i = -1;
		      } else
                          /* obj class != 0 to force generation of a specific item */
                          yyval.i = SP_OBJ_PACK(m, 1);
                      Free(yypvt[-0].map);
		  } break;
case 337:
# line 2335 "lev_comp.y"
{
			if (check_object_char((char) yypvt[-0].i))
			    yyval.i = SP_OBJ_PACK(-1, yypvt[-0].i);
			else {
			    lc_error("Unknown object class '%c'!", yypvt[-0].i);
			    yyval.i = -1;
			}
		  } break;
case 338:
# line 2344 "lev_comp.y"
{
		      long m = get_object_id(yypvt[-1].map, (char) yypvt[-3].i);
		      if (m == ERR) {
			  lc_error("Unknown object ('%c', \"%s\")!", yypvt[-3].i, yypvt[-1].map);
			  yyval.i = -1;
		      } else
			  yyval.i = SP_OBJ_PACK(m, yypvt[-3].i);
                      Free(yypvt[-1].map);
		  } break;
case 339:
# line 2354 "lev_comp.y"
{
		      yyval.i = -1;
		  } break;
case 340:
# line 2360 "lev_comp.y"
{ } break;
case 341:
# line 2362 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_ADD));
		  } break;
case 342:
# line 2368 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(yypvt[-0].i));
		  } break;
case 343:
# line 2372 "lev_comp.y"
{
		      is_inconstant_number = 1;
		  } break;
case 344:
# line 2376 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(yypvt[-1].i));
		  } break;
case 345:
# line 2380 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_INT);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		      is_inconstant_number = 1;
		  } break;
case 346:
# line 2388 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-3].map,
					SPOVAR_INT | SPOVAR_ARRAY);
		      vardef_used(vardefs, yypvt[-3].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-3].map));
		      Free(yypvt[-3].map);
		      is_inconstant_number = 1;
		  } break;
case 347:
# line 2397 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_ADD));
		  } break;
case 348:
# line 2401 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_SUB));
		  } break;
case 349:
# line 2405 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_MUL));
		  } break;
case 350:
# line 2409 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_DIV));
		  } break;
case 351:
# line 2413 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_MATH_MOD));
		  } break;
case 352:
# line 2416 "lev_comp.y"
{ } break;
case 353:
# line 2420 "lev_comp.y"
{
		      if (!strcmp("int", yypvt[-0].map) || !strcmp("integer", yypvt[-0].map)) {
			  yyval.i = (int)'i';
		      } else
			  lc_error("Unknown function parameter type '%s'", yypvt[-0].map);
		  } break;
case 354:
# line 2427 "lev_comp.y"
{
		      if (!strcmp("str", yypvt[-0].map) || !strcmp("string", yypvt[-0].map)) {
			  yyval.i = (int)'s';
		      } else
			  lc_error("Unknown function parameter type '%s'", yypvt[-0].map);
		  } break;
case 355:
# line 2436 "lev_comp.y"
{
		      struct lc_funcdefs_parm *tmp = New(struct lc_funcdefs_parm);

		      if (!curr_function) {
			  lc_error("Function parameters outside function definition.");
		      } else if (!tmp) {
			  lc_error("Could not alloc function params.");
		      } else {
			  long vt = SPOVAR_NULL;

			  tmp->name = strdup(yypvt[-2].map);
			  tmp->parmtype = (char) yypvt[-0].i;
			  tmp->next = curr_function->params;
			  curr_function->params = tmp;
			  curr_function->n_params++;
			  switch (tmp->parmtype) {
			  case 'i':
                              vt = SPOVAR_INT;
                              break;
			  case 's':
                              vt = SPOVAR_STRING;
                              break;
			  default:
                              lc_error("Unknown func param conversion.");
                              break;
			  }
			  vardefs = add_vardef_type( vardefs, yypvt[-2].map, vt);
		      }
		      Free(yypvt[-2].map);
		  } break;
case 360:
# line 2477 "lev_comp.y"
{
			      yyval.i = (int)'i';
			  } break;
case 361:
# line 2481 "lev_comp.y"
{
			      yyval.i = (int)'s';
			  } break;
case 362:
# line 2488 "lev_comp.y"
{
			      char tmpbuf[2];
			      tmpbuf[0] = (char) yypvt[-0].i;
			      tmpbuf[1] = '\0';
			      yyval.map = strdup(tmpbuf);
			  } break;
case 363:
# line 2495 "lev_comp.y"
{
			      long len = strlen( yypvt[-2].map );
			      char *tmp = (char *) alloc(len + 2);
			      sprintf(tmp, "%c%s", (char) yypvt[-0].i, yypvt[-2].map );
			      Free( yypvt[-2].map );
			      yyval.map = tmp;
			  } break;
case 364:
# line 2505 "lev_comp.y"
{
			      yyval.map = strdup("");
			  } break;
case 365:
# line 2509 "lev_comp.y"
{
			      char *tmp = strdup( yypvt[-0].map );
			      Free( yypvt[-0].map );
			      yyval.map = tmp;
			  } break;
case 366:
# line 2517 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_POINT));
		  } break;
case 367:
# line 2521 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_RECT));
		  } break;
case 368:
# line 2525 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_FILLRECT));
		  } break;
case 369:
# line 2529 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_LINE));
		  } break;
case 370:
# line 2533 "lev_comp.y"
{
		      /* randline (x1,y1),(x2,y2), roughness */
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_RNDLINE));
		  } break;
case 371:
# line 2538 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(W_ANY, SPO_SEL_GROW));
		  } break;
case 372:
# line 2542 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(yypvt[-3].i, SPO_SEL_GROW));
		  } break;
case 373:
# line 2546 "lev_comp.y"
{
		      add_opvars(splev, "iio",
			     VA_PASS3(yypvt[-3].i, SPOFILTER_PERCENT, SPO_SEL_FILTER));
		  } break;
case 374:
# line 2551 "lev_comp.y"
{
		      add_opvars(splev, "io",
			       VA_PASS2(SPOFILTER_SELECTION, SPO_SEL_FILTER));
		  } break;
case 375:
# line 2556 "lev_comp.y"
{
		      add_opvars(splev, "io",
				 VA_PASS2(SPOFILTER_MAPCHAR, SPO_SEL_FILTER));
		  } break;
case 376:
# line 2561 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_FLOOD));
		  } break;
case 377:
# line 2565 "lev_comp.y"
{
		      add_opvars(splev, "oio",
				 VA_PASS3(SPO_COPY, 1, SPO_SEL_ELLIPSE));
		  } break;
case 378:
# line 2570 "lev_comp.y"
{
		      add_opvars(splev, "oio",
				 VA_PASS3(SPO_COPY, yypvt[-1].i, SPO_SEL_ELLIPSE));
		  } break;
case 379:
# line 2575 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(1, SPO_SEL_ELLIPSE));
		  } break;
case 380:
# line 2579 "lev_comp.y"
{
		      add_opvars(splev, "io", VA_PASS2(yypvt[-1].i, SPO_SEL_ELLIPSE));
		  } break;
case 381:
# line 2583 "lev_comp.y"
{
		      add_opvars(splev, "iio",
				 VA_PASS3(yypvt[-5].i, yypvt[-11].i, SPO_SEL_GRADIENT));
		  } break;
case 382:
# line 2588 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_COMPLEMENT));
		  } break;
case 383:
# line 2592 "lev_comp.y"
{
		      check_vardef_type(vardefs, yypvt[-0].map, SPOVAR_SEL);
		      vardef_used(vardefs, yypvt[-0].map);
		      add_opvars(splev, "v", VA_PASS1(yypvt[-0].map));
		      Free(yypvt[-0].map);
		  } break;
case 384:
# line 2599 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 385:
# line 2605 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 386:
# line 2609 "lev_comp.y"
{
		      add_opvars(splev, "o", VA_PASS1(SPO_SEL_ADD));
		  } break;
case 387:
# line 2615 "lev_comp.y"
{
		      add_opvars(splev, "iio",
				 VA_PASS3(yypvt[-0].dice.num, yypvt[-0].dice.die, SPO_DICE));
		  } break;
case 391:
# line 2627 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(yypvt[-0].i));
		  } break;
case 392:
# line 2631 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(yypvt[-0].i));
		  } break;
case 393:
# line 2635 "lev_comp.y"
{
		      add_opvars(splev, "i", VA_PASS1(yypvt[-0].i));
		  } break;
case 394:
# line 2639 "lev_comp.y"
{
		      /* nothing */
		  } break;
case 403:
# line 2661 "lev_comp.y"
{
			yyval.lregn = yypvt[-0].lregn;
		  } break;
case 404:
# line 2665 "lev_comp.y"
{
			if (yypvt[-7].i <= 0 || yypvt[-7].i >= COLNO)
			    lc_error(
                          "Region (%ld,%ld,%ld,%ld) out of level range (x1)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			else if (yypvt[-5].i < 0 || yypvt[-5].i >= ROWNO)
			    lc_error(
                          "Region (%ld,%ld,%ld,%ld) out of level range (y1)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			else if (yypvt[-3].i <= 0 || yypvt[-3].i >= COLNO)
			    lc_error(
                          "Region (%ld,%ld,%ld,%ld) out of level range (x2)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			else if (yypvt[-1].i < 0 || yypvt[-1].i >= ROWNO)
			    lc_error(
                          "Region (%ld,%ld,%ld,%ld) out of level range (y2)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			yyval.lregn.x1 = yypvt[-7].i;
			yyval.lregn.y1 = yypvt[-5].i;
			yyval.lregn.x2 = yypvt[-3].i;
			yyval.lregn.y2 = yypvt[-1].i;
			yyval.lregn.area = 1;
		  } break;
case 405:
# line 2691 "lev_comp.y"
{
/* This series of if statements is a hack for MSC 5.1.  It seems that its
   tiny little brain cannot compile if these are all one big if statement. */
			if (yypvt[-7].i < 0 || yypvt[-7].i > (int) max_x_map)
			    lc_error(
                            "Region (%ld,%ld,%ld,%ld) out of map range (x1)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			else if (yypvt[-5].i < 0 || yypvt[-5].i > (int) max_y_map)
			    lc_error(
                            "Region (%ld,%ld,%ld,%ld) out of map range (y1)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			else if (yypvt[-3].i < 0 || yypvt[-3].i > (int) max_x_map)
			    lc_error(
                            "Region (%ld,%ld,%ld,%ld) out of map range (x2)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			else if (yypvt[-1].i < 0 || yypvt[-1].i > (int) max_y_map)
			    lc_error(
                            "Region (%ld,%ld,%ld,%ld) out of map range (y2)!",
                                     yypvt[-7].i, yypvt[-5].i, yypvt[-3].i, yypvt[-1].i);
			yyval.lregn.area = 0;
			yyval.lregn.x1 = yypvt[-7].i;
			yyval.lregn.y1 = yypvt[-5].i;
			yyval.lregn.x2 = yypvt[-3].i;
			yyval.lregn.y2 = yypvt[-1].i;
		  } break; 
		}
		goto yystack;  /* stack new state and value */

	}
